@@ -21,11 +21,18 @@ approach
 
 The results are returned in a :class:`pykeen.pipeline.PipelineResult` instance, which has
 attributes for the trained model, the training loop, and the evaluation.
+
+PyKEEN has a function :func:`pykeen.env` that magically prints relevant version information
+about PyTorch, CUDA, and your operating system that can be used for debugging.
+If you're in a Jupyter notebook, it will be pretty printed as an HTML table.
+
+>>> import pykeen
+>>> pykeen.env()
 """
 
 import logging
 
-from .version import get_version  # noqa: F401
+from .version import env, get_git_branch, get_git_hash, get_version  # noqa: F401
 
 # This will set the global logging level to info to ensure that info messages are shown in all parts of the software.
 logging.getLogger(__name__).setLevel(logging.INFO)
