@@ -5,35 +5,40 @@
 Get a summary with ``python -m pykeen.datasets.hetionet``
 """
 
-import logging
-
 import click
+from docdata import parse_docdata
+from more_click import verbose_option
 
 from .base import SingleTabbedDataset
 from ..typing import TorchRandomHint
 
 __all__ = [
-    'Hetionet',
+    "Hetionet",
 ]
 
-URL = 'https://github.com/hetio/hetionet/raw/master/hetnet/tsv/hetionet-v1.0-edges.sif.gz'
+URL = "https://github.com/hetio/hetionet/raw/master/hetnet/tsv/hetionet-v1.0-edges.sif.gz"
 
 
+@parse_docdata
 class Hetionet(SingleTabbedDataset):
-    """The Hetionet dataset is a large biological network.
+    """The Hetionet dataset from [himmelstein2017]_.
 
     In its publication [himmelstein2017]_, it is demonstrated to be useful for link prediction in drug repositioning
     and made publicly available through its `GitHub repository <https://github.com/hetio/hetionet>`_ in several formats.
     The link prediction algorithm showcased does not rely on embeddings, which leaves room for interesting comparison.
     One such comparison was made during the master's thesis of Lingling Xu [xu2019]_.
-
-    For reproducibility, the random_state argument is set by default to 0. For permutation studies, you can change
-    this.
-
-    .. [himmelstein2017] Himmelstein, D. S., *et al* (2017). `Systematic integration of biomedical knowledge
-       prioritizes drugs for repurposing <https://doi.org/10.7554/eLife.26726>`_. ELife, 6.
-    .. [xu2019] Xu, L (2019) `A Comparison of Learned and Engineered Features in Network-Based Drug Repositioning
-       <https://github.com/lingling93/master_thesis_drugrelink>`_. Master's Thesis.
+    ---
+    name: Hetionet
+    citation:
+        author: Himmelstein
+        year: 2017
+        link: https://doi.org/10.7554/eLife.26726
+        github: hetio/hetionet
+    single: true
+    statistics:
+        entities: 45158
+        relations: 24
+        triples: 2250197
     """
 
     def __init__(
@@ -57,11 +62,11 @@ class Hetionet(SingleTabbedDataset):
 
 
 @click.command()
+@verbose_option
 def _main():
     ds = Hetionet()
-    click.echo(ds.summary_str())
+    ds.summarize()
 
 
-if __name__ == '__main__':
-    logging.basicConfig(level=logging.INFO)
+if __name__ == "__main__":
     _main()
