@@ -5,25 +5,22 @@
 Get a summary with ``python -m pykeen.datasets.openbiolink``
 """
 
-import logging
-
 import click
+from docdata import parse_docdata
+from more_click import verbose_option
 
 from .base import PackedZipRemoteDataset
 
 __all__ = [
-    'OpenBioLink',
-    'OpenBioLinkF1',
-    'OpenBioLinkF2',
-    'OpenBioLinkLQ',
+    "OpenBioLink",
+    "OpenBioLinkLQ",
 ]
 
-HQ_URL = 'https://samwald.info/res/OpenBioLink_2020_final/HQ_DIR.zip'
-F1_URL = 'https://github.com/PyKEEN/pykeen-openbiolink-benchmark/raw/master/filter_1/openbiolink_f1.zip'
-F2_URL = 'https://github.com/PyKEEN/pykeen-openbiolink-benchmark/raw/master/filter_2/openbiolink_f2.zip'
-LQ_URL = 'https://samwald.info/res/OpenBioLink_2020_final/ALL_DIR.zip'
+HQ_URL = "https://samwald.info/res/OpenBioLink_2020_final/HQ_DIR.zip"
+LQ_URL = "https://samwald.info/res/OpenBioLink_2020_final/ALL_DIR.zip"
 
 
+@parse_docdata
 class OpenBioLink(PackedZipRemoteDataset):
     """The OpenBioLink dataset.
 
@@ -32,8 +29,20 @@ class OpenBioLink(PackedZipRemoteDataset):
     at https://github.com/openbiolink/openbiolink and published in [breit2020]_. There are four
     available datasets - this class represents the high quality, directed set.
 
-    .. [breit2020] Breit, A. (2020) `OpenBioLink: A benchmarking framework for large-scale biomedical link
-       prediction <https://doi.org/10.1093/bioinformatics/btaa274>`_, *Bioinformatics*
+    ---
+    name: OpenBioLink
+    citation:
+        author: Breit
+        year: 2020
+        link: https://doi.org/10.1093/bioinformatics/btaa274
+        github: openbiolink/openbiolink
+    statistics:
+        entities: 180992
+        relations: 28
+        training: 4192002
+        testing: 183011
+        validation: 188394
+        triples: 4563407
     """
 
     def __init__(self, create_inverse_triples: bool = False, **kwargs):
@@ -44,57 +53,34 @@ class OpenBioLink(PackedZipRemoteDataset):
         """
         super().__init__(
             url=HQ_URL,
-            name='HQ_DIR.zip',
-            relative_training_path='HQ_DIR/train_test_data/train_sample.csv',
-            relative_testing_path='HQ_DIR/train_test_data/test_sample.csv',
-            relative_validation_path='HQ_DIR/train_test_data/val_sample.csv',
-            create_inverse_triples=create_inverse_triples,
-            **kwargs,
-        )
-
-
-class OpenBioLinkF1(PackedZipRemoteDataset):
-    """The PyKEEN First Filtered OpenBioLink 2020 Dataset."""
-
-    def __init__(self, create_inverse_triples: bool = False, **kwargs):
-        """Initialize the OpenBioLink (Filter-1) dataset.
-
-        :param create_inverse_triples: Should inverse triples be created? Defaults to false.
-        :param kwargs: keyword arguments passed to :class:`pykeen.datasets.base.PackedZipRemoteDataset`.
-        """
-        super().__init__(
-            url=F1_URL,
-            name='openbiolink_f1.zip',
-            relative_training_path='train.tsv',
-            relative_testing_path='test.tsv',
-            relative_validation_path='val.tsv',
-            create_inverse_triples=create_inverse_triples,
-            **kwargs,
-        )
-
-
-class OpenBioLinkF2(PackedZipRemoteDataset):
-    """The PyKEEN Second Filtered OpenBioLink 2020 Dataset."""
-
-    def __init__(self, create_inverse_triples: bool = False, **kwargs):
-        """Initialize the OpenBioLink (Filter-2) dataset.
-
-        :param create_inverse_triples: Should inverse triples be created? Defaults to false.
-        :param kwargs: keyword arguments passed to :class:`pykeen.datasets.base.PackedZipRemoteDataset`.
-        """
-        super().__init__(
-            url=F2_URL,
-            name='openbiolink_f2.zip',
-            relative_training_path='train.tsv',
-            relative_testing_path='test.tsv',
-            relative_validation_path='val.tsv',
+            name="HQ_DIR.zip",
+            relative_training_path="HQ_DIR/train_test_data/train_sample.csv",
+            relative_testing_path="HQ_DIR/train_test_data/test_sample.csv",
+            relative_validation_path="HQ_DIR/train_test_data/val_sample.csv",
             create_inverse_triples=create_inverse_triples,
             **kwargs,
         )
 
 
+@parse_docdata
 class OpenBioLinkLQ(PackedZipRemoteDataset):
-    """The low-quality variant of the OpenBioLink dataset."""
+    """The low-quality variant of the OpenBioLink dataset.
+
+    ---
+    name: OpenBioLink
+    citation:
+        author: Breit
+        year: 2020
+        link: https://doi.org/10.1093/bioinformatics/btaa274
+        github: openbiolink/openbiolink
+    statistics:
+        entities: 480876
+        relations: 32
+        training: 25508954
+        testing: 679934
+        validation: 1132001
+        triples: 27320889
+    """
 
     def __init__(self, create_inverse_triples: bool = False, **kwargs):
         """Initialize the OpenBioLink (low quality) dataset.
@@ -104,21 +90,21 @@ class OpenBioLinkLQ(PackedZipRemoteDataset):
         """
         super().__init__(
             url=LQ_URL,
-            name='ALL_DIR.zip',
-            relative_training_path='ALL_DIR/train_test_data/train_sample.csv',
-            relative_testing_path='ALL_DIR/train_test_data/test_sample.csv',
-            relative_validation_path='ALL_DIR/train_test_data/val_sample.csv',
+            name="ALL_DIR.zip",
+            relative_training_path="ALL_DIR/train_test_data/train_sample.csv",
+            relative_testing_path="ALL_DIR/train_test_data/test_sample.csv",
+            relative_validation_path="ALL_DIR/train_test_data/val_sample.csv",
             create_inverse_triples=create_inverse_triples,
             **kwargs,
         )
 
 
 @click.command()
+@verbose_option
 def _main():
-    ds = OpenBioLink()
-    click.echo(ds.summary_str())
+    for cls in [OpenBioLink, OpenBioLinkLQ]:
+        cls().summarize()
 
 
-if __name__ == '__main__':
-    logging.basicConfig(level=logging.INFO)
+if __name__ == "__main__":
     _main()
