@@ -9,6 +9,7 @@ from abc import ABC, abstractmethod
 from contextlib import contextmanager
 from dataclasses import dataclass
 from math import ceil
+from textwrap import dedent
 from typing import Any, Collection, Iterable, List, Mapping, Optional, Tuple, Union, cast
 
 import numpy as np
@@ -17,18 +18,23 @@ from dataclasses_json import DataClassJsonMixin
 from tqdm.autonotebook import tqdm
 
 from ..models import Model
-from ..triples.utils import get_entities
+from ..triples.triples_factory import restrict_triples
+from ..triples.utils import get_entities, get_relations
 from ..typing import MappedTriples
 from ..utils import (
-    is_cuda_oom_error, is_cudnn_error, is_nonzero_larger_than_maxint_error, normalize_string,
+    format_relative_comparison,
+    is_cuda_oom_error,
+    is_cudnn_error,
+    is_nonzero_larger_than_maxint_error,
+    normalize_string,
     split_list_in_batches_iter,
 )
 
 __all__ = [
-    'Evaluator',
-    'MetricResults',
-    'filter_scores_',
-    'evaluate',
+    "Evaluator",
+    "MetricResults",
+    "filter_scores_",
+    "evaluate",
 ]
 
 logger = logging.getLogger(__name__)
@@ -48,7 +54,11 @@ class MetricResults(DataClassJsonMixin):
     """Results from computing metrics."""
 
     def get_metric(self, name: str) -> float:
-        """Get the given metric from the results."""
+        """Get the given metric from the results.
+
+        :param name: The name of the metric
+        :returns: The value for the metric
+        """
         raise NotImplementedError
 
     def to_flat_dict(self) -> Mapping[str, Any]:
@@ -72,6 +82,15 @@ class Evaluator(ABC):
         slice_size: Optional[int] = None,
         automatic_memory_optimization: bool = True,
     ):
+        """Initialize the evaluator.
+
+        :param filtered: Should filtered evaluation be performed?
+        :param requires_positive_mask: Does the evaluator need access to the masks?
+        :param batch_size: >0. Evaluation batch size.
+        :param slice_size: >0. The divisor for the scoring function when using slicing
+        :param automatic_memory_optimization: Whether to automatically optimize the sub-batch size during
+            evaluation with regards to the hardware at hand.
+        """
         self.filtered = filtered
         self.requires_positive_mask = requires_positive_mask
         self.batch_size = batch_size
@@ -127,7 +146,7 @@ class Evaluator(ABC):
     def evaluate(
         self,
         model: Model,
-        mapped_triples: Optional[MappedTriples] = None,
+        mapped_triples: MappedTriples,
         batch_size: Optional[int] = None,
         slice_size: Optional[int] = None,
         device: Optional[torch.device] = None,
@@ -135,14 +154,12 @@ class Evaluator(ABC):
         tqdm_kwargs: Optional[Mapping[str, str]] = None,
         restrict_entities_to: Optional[torch.LongTensor] = None,
         do_time_consuming_checks: bool = True,
+        additional_filter_triples: Union[None, MappedTriples, List[MappedTriples]] = None,
     ) -> MetricResults:
         """Run :func:`pykeen.evaluation.evaluate` with this evaluator."""
-        if mapped_triples is None:
-            mapped_triples = model.triples_factory.mapped_triples
-
         if batch_size is None and self.automatic_memory_optimization:
             # Using automatic memory optimization on CPU may result in undocumented crashes due to OS' OOM killer.
-            if model.device.type == 'cpu':
+            if model.device.type == "cpu":
                 logger.info(
                     "Currently automatic memory optimization only supports GPUs, but you're using a CPU. "
                     "Therefore, the batch_size will be set to the default value.",
@@ -151,6 +168,7 @@ class Evaluator(ABC):
                 batch_size, slice_size = self.batch_and_slice(
                     model=model,
                     mapped_triples=mapped_triples,
+                    additional_filter_triples=additional_filter_triples,
                     batch_size=batch_size,
                     device=device,
                     use_tqdm=False,
@@ -166,6 +184,7 @@ class Evaluator(ABC):
 
         rv = evaluate(
             model=model,
+            additional_filter_triples=additional_filter_triples,
             mapped_triples=mapped_triples,
             evaluators=self,
             batch_size=batch_size,
@@ -189,6 +208,7 @@ class Evaluator(ABC):
         use_tqdm: bool = False,
         restrict_entities_to: Optional[torch.LongTensor] = None,
         do_time_consuming_checks: bool = True,
+        additional_filter_triples: Union[None, MappedTriples, List[MappedTriples]] = None,
     ) -> Tuple[int, Optional[int]]:
         """Find the maximum possible batch_size and slice_size for evaluation with the current setting.
 
@@ -212,6 +232,9 @@ class Evaluator(ABC):
             Should a progress bar be displayed?
         :param restrict_entities_to:
             Whether to restrict the evaluation to certain entities of interest.
+        :param additional_filter_triples:
+            Additional true triples to filter out during filtered evaluation. Only needed if the evaluator is in
+            filtered mode.
 
         :return:
             Maximum possible batch size and, if necessary, the slice_size, which defaults to None.
@@ -220,9 +243,10 @@ class Evaluator(ABC):
             If it is not possible to evaluate the model on the hardware at hand with the given parameters.
         """
         batch_size, evaluated_once = self._param_size_search(
-            key='batch_size',
+            key="batch_size",
             start_value=batch_size,
             model=model,
+            additional_filter_triples=additional_filter_triples,
             mapped_triples=mapped_triples,
             device=device,
             use_tqdm=use_tqdm,
@@ -235,11 +259,12 @@ class Evaluator(ABC):
 
         # We need to try slicing, if the evaluation for the batch_size search never succeeded
         slice_size, evaluated_once = self._param_size_search(
-            key='slice_size',
+            key="slice_size",
             # Since the batch_size search with size 1, i.e. one tuple ((h, r) or (r, t)) scored on all entities,
             # must have failed to start slice_size search, we start with trying half the entities.
             start_value=ceil(model.num_entities / 2),
             model=model,
+            additional_filter_triples=additional_filter_triples,
             mapped_triples=mapped_triples,
             device=device,
             use_tqdm=use_tqdm,
@@ -261,36 +286,38 @@ class Evaluator(ABC):
         use_tqdm: bool = False,
         restrict_entities_to: Optional[torch.LongTensor] = None,
         do_time_consuming_checks: bool = True,
+        additional_filter_triples: Union[None, MappedTriples, List[MappedTriples]] = None,
     ) -> Tuple[int, bool]:
         values_dict = {}
         maximum_triples = mapped_triples.shape[0]
-        if key == 'batch_size':
+        if key == "batch_size":
             if start_value is None:
                 start_value = 256
             if start_value > maximum_triples:
                 start_value = maximum_triples
             values_dict[key] = start_value
-            values_dict['slice_size'] = None
-        elif key == 'slice_size':
+            values_dict["slice_size"] = None
+        elif key == "slice_size":
             if start_value is None:
                 start_value = ceil(model.num_entities / 2)
             self._check_slicing_availability(model, batch_size=1)
             values_dict[key] = start_value
-            values_dict['batch_size'] = 1
+            values_dict["batch_size"] = 1
         else:
-            raise AttributeError(f'The parameter {key} is unknown.')
+            raise AttributeError(f"The parameter {key} is unknown.")
 
         reached_max = False
         evaluated_once = False
-        logger.info(f'Starting {key} search for evaluation now...')
+        logger.info(f"Starting {key} search for evaluation now...")
         while True:
-            logger.debug(f'Trying {key}={values_dict[key]}')
+            logger.debug(f"Trying {key}={values_dict[key]}")
             try:
                 # The cache of the previous run has to be freed to allow accurate memory availability estimates
                 gc.collect()
                 torch.cuda.empty_cache()
                 evaluate(
                     model=model,
+                    additional_filter_triples=additional_filter_triples,
                     mapped_triples=mapped_triples,
                     evaluators=self,
                     only_size_probing=True,
@@ -299,8 +326,8 @@ class Evaluator(ABC):
                     use_tqdm=use_tqdm,
                     restrict_entities_to=restrict_entities_to,
                     do_time_consuming_checks=do_time_consuming_checks,
-                    batch_size=values_dict.get('batch_size'),
-                    slice_size=values_dict.get('slice_size'),
+                    batch_size=values_dict.get("batch_size"),
+                    slice_size=values_dict.get("slice_size"),
                 )
                 evaluated_once = True
             except RuntimeError as runtime_error:
@@ -327,18 +354,18 @@ class Evaluator(ABC):
                 values_dict[key] //= 2  # type: ignore
                 reached_max = True
                 if evaluated_once:
-                    logger.info(f'Concluded {key} search with batch_size={values_dict[key]}.')
+                    logger.info(f"Concluded {key} search with batch_size={values_dict[key]}.")
                     break
                 else:
-                    logger.debug(f'The {key} {values_dict[key]} was too big, trying less now')
+                    logger.debug(f"The {key} {values_dict[key]} was too big, trying less now")
             else:
                 # The cache of the previous run has to be freed to allow accurate memory availability estimates
                 gc.collect()
                 torch.cuda.empty_cache()
-                if not reached_max and values_dict['batch_size'] < maximum_triples:
+                if not reached_max and values_dict["batch_size"] < maximum_triples:
                     values_dict[key] *= 2  # type: ignore
                 else:
-                    logger.info(f'Concluded {key} search with batch_size={values_dict[key]}.')
+                    logger.info(f"Concluded {key} search with batch_size={values_dict[key]}.")
                     break
 
         return cast(Tuple[int, bool], (values_dict[key], evaluated_once))
@@ -346,15 +373,19 @@ class Evaluator(ABC):
     @staticmethod
     def _check_slicing_availability(model: Model, batch_size: int) -> None:
         # Test if slicing is implemented for the required functions of this model
-        if model.triples_factory.create_inverse_triples:
+        if model.use_inverse_triples:
             if not model.can_slice_t:
-                raise MemoryError(f"The current model can't be evaluated on this hardware with these parameters, as "
-                                  f"evaluation batch_size={batch_size} is too big and slicing is not implemented for "
-                                  f"this model yet.")
+                raise MemoryError(
+                    f"The current model can't be evaluated on this hardware with these parameters, as "
+                    f"evaluation batch_size={batch_size} is too big and slicing is not implemented for "
+                    f"this model yet."
+                )
         elif not model.can_slice_t or not model.can_slice_h:
-            raise MemoryError(f"The current model can't be evaluated on this hardware with these parameters, as "
-                              f"evaluation batch_size={batch_size} is too big and slicing is not implemented for this "
-                              f"model yet.")
+            raise MemoryError(
+                f"The current model can't be evaluated on this hardware with these parameters, as "
+                f"evaluation batch_size={batch_size} is too big and slicing is not implemented for this "
+                f"model yet."
+            )
 
 
 def create_sparse_positive_filter_(
@@ -388,8 +419,8 @@ def create_sparse_positive_filter_(
     """
     if filter_col not in {0, 2}:
         raise NotImplementedError(
-            'This code has only been written for updating head (filter_col=0) or '
-            f'tail (filter_col=2) mask, but filter_col={filter_col} was given.',
+            "This code has only been written for updating head (filter_col=0) or "
+            f"tail (filter_col=2) mask, but filter_col={filter_col} was given.",
         )
 
     if relation_filter is None:
@@ -398,11 +429,11 @@ def create_sparse_positive_filter_(
 
     # Split batch
     other_col = 2 - filter_col
-    entities = hrt_batch[:, other_col:other_col + 1]
+    entities = hrt_batch[:, other_col : other_col + 1]
 
-    entity_filter_test = (all_pos_triples[:, other_col:other_col + 1]).view(1, -1) == entities
+    entity_filter_test = (all_pos_triples[:, other_col : other_col + 1]).view(1, -1) == entities
     filter_batch = (entity_filter_test & relation_filter).nonzero(as_tuple=False)
-    filter_batch[:, 1] = all_pos_triples[:, filter_col:filter_col + 1].view(1, -1)[:, filter_batch[:, 1]]
+    filter_batch[:, 1] = all_pos_triples[:, filter_col : filter_col + 1].view(1, -1)[:, filter_batch[:, 1]]
 
     return filter_batch, relation_filter
 
@@ -443,14 +474,13 @@ def filter_scores_(
     batch_size, num_entities = scores.shape
 
     # Set all filtered triples to NaN to ensure their exclusion in subsequent calculations
-    scores[filter_batch[:, 0], filter_batch[:, 1]] = float('nan')
+    scores[filter_batch[:, 0], filter_batch[:, 1]] = float("nan")
 
     # Warn if all entities will be filtered
     # (scores != scores) yields true for all NaN instances (IEEE 754), thus allowing to count the filtered triples.
     if ((scores != scores).sum(dim=1) == num_entities).any():
         logger.warning(
-            "User selected filtered metric computation, but all corrupted triples exists also as positive "
-            "triples",
+            "User selected filtered metric computation, but all corrupted triples exists also as positive " "triples",
         )
 
     return scores
@@ -467,8 +497,11 @@ def evaluate(
     squeeze: bool = True,
     use_tqdm: bool = True,
     tqdm_kwargs: Optional[Mapping[str, str]] = None,
-    restrict_entities_to: Optional[torch.LongTensor] = None,
+    restrict_entities_to: Optional[Collection[int]] = None,
+    restrict_relations_to: Optional[Collection[int]] = None,
     do_time_consuming_checks: bool = True,
+    additional_filter_triples: Union[None, MappedTriples, List[MappedTriples]] = None,
+    pre_filtered_triples: bool = True,
 ) -> Union[MetricResults, List[MetricResults]]:
     """Evaluate metrics for model on mapped triples.
 
@@ -497,17 +530,30 @@ def evaluate(
         Return a single instance of :class:`MetricResults` if only one evaluator was given.
     :param use_tqdm:
         Should a progress bar be displayed?
+    :param tqdm_kwargs:
+        Additional keyword based arguments passed to the progress bar.
     :param restrict_entities_to:
         Optionally restrict the evaluation to the given entity IDs. This may be useful if one is only interested in a
         part of the entities, e.g. due to type constraints, but wants to train on all available data. For ranking the
         entities, we still compute all scores for all possible replacement entities to avoid irregular access patterns
-        which might decrease performance, but the scores with afterwards be filtered to only keep those of interest.
-        If provided, we assume that the triples are already filtered, such that it only contains the entities of
-        interest.
+        which might decrease performance, but the scores will afterwards be filtered to only keep those of interest.
+        If provided, we assume by default that the triples are already filtered, such that it only contains the
+        entities of interest. To explicitly filter within this method, pass `pre_filtered_triples=False`.
+    :param restrict_relations_to:
+        Optionally restrict the evaluation to the given relation IDs. This may be useful if one is only interested in a
+        part of the relations, e.g. due to relation types, but wants to train on all available data. If provided, we
+        assume by default that the triples are already filtered, such that it only contains the relations of interest.
+        To explicitly filter within this method, pass `pre_filtered_triples=False`.
     :param do_time_consuming_checks:
         Whether to perform some time consuming checks on the provided arguments. Currently, this encompasses:
-        - If restrict_entities_to is not None, check whether the triples have been filtered.
-        Disabling this option can accelerate the method.
+        - If restrict_entities_to or restrict_relations_to is not None, check whether the triples have been filtered.
+        Disabling this option can accelerate the method. Only effective if pre_filtered_triples is set to True.
+    :param pre_filtered_triples:
+        Whether the triples have been pre-filtered to adhere to restrict_entities_to / restrict_relations_to. When set
+        to True, and the triples have *not* been filtered, the results may be invalid. Pre-filtering the triples
+        accelerates this method, and is recommended when evaluating multiple times on the same set of triples.
+    :param additional_filtered_triples:
+        Additional true triples to filter out during filtered evaluation.
     """
     if isinstance(evaluators, Evaluator):  # upgrade a single evaluator to a list
         evaluators = [evaluators]
@@ -515,12 +561,33 @@ def evaluate(
     start = timeit.default_timer()
 
     # verify that the triples have been filtered
-    if restrict_entities_to is not None and do_time_consuming_checks:
-        present_entity_ids = get_entities(triples=mapped_triples)
-        unwanted = present_entity_ids.difference(restrict_entities_to.tolist())
-        if len(unwanted) > 0:
-            raise ValueError(f'mapped_triples contains IDs of entities which are not contained in restrict_entities_to:'
-                             f'{unwanted}. This will invalidate the evaluation results.')
+    if pre_filtered_triples and do_time_consuming_checks:
+        if restrict_entities_to is not None:
+            present_entity_ids = get_entities(triples=mapped_triples)
+            unwanted = present_entity_ids.difference(restrict_entities_to)
+            if len(unwanted) > 0:
+                raise ValueError(
+                    f"mapped_triples contains IDs of entities which are not contained in restrict_entities_to:"
+                    f"{unwanted}. This will invalidate the evaluation results.",
+                )
+        if restrict_relations_to is not None:
+            present_relation_ids = get_relations(triples=mapped_triples)
+            unwanted = present_relation_ids.difference(restrict_relations_to)
+            if len(unwanted):
+                raise ValueError(
+                    f"mapped_triples contains IDs of relations which are not contained in restrict_relations_to:"
+                    f"{unwanted}. This will invalidate the evaluation results.",
+                )
+
+    # Filter triples if necessary
+    if not pre_filtered_triples and (restrict_entities_to is not None or restrict_relations_to is not None):
+        old_num_triples = mapped_triples.shape[0]
+        mapped_triples = restrict_triples(
+            mapped_triples=mapped_triples,
+            entities=restrict_entities_to,
+            relations=restrict_relations_to,
+        )
+        logger.info(f"keeping {format_relative_comparison(mapped_triples.shape[0], old_num_triples)} triples.")
 
     # Send to device
     if device is not None:
@@ -543,7 +610,29 @@ def evaluate(
 
     # Prepare for result filtering
     if filtering_necessary or positive_masks_required:
-        all_pos_triples = torch.cat([model.triples_factory.mapped_triples, mapped_triples], dim=0)
+        if additional_filter_triples is None:
+            logger.warning(
+                dedent(
+                    """\
+                The filtered setting was enabled, but there were no `additional_filter_triples`
+                given. This means you probably forgot to pass (at least) the training triples. Try:
+
+                    additional_filter_triples=[dataset.training.mapped_triples]
+
+                Or if you want to use the Bordes et al. (2013) approach to filtering, do:
+
+                    additional_filter_triples=[
+                        dataset.training.mapped_triples,
+                        dataset.validation.mapped_triples,
+                    ]
+            """
+                )
+            )
+            all_pos_triples = mapped_triples
+        elif isinstance(additional_filter_triples, (list, tuple)):
+            all_pos_triples = torch.cat([*additional_filter_triples, mapped_triples], dim=0)
+        else:
+            all_pos_triples = torch.cat([additional_filter_triples, mapped_triples], dim=0)
         all_pos_triples = all_pos_triples.to(device=device)
     else:
         all_pos_triples = None
@@ -566,23 +655,23 @@ def evaluate(
 
     # Disable gradient tracking
     _tqdm_kwargs = dict(
-        desc=f'Evaluating on {model.device}',
+        desc=f"Evaluating on {model.device}",
         total=num_triples,
-        unit='triple',
+        unit="triple",
         unit_scale=True,
         # Choosing no progress bar (use_tqdm=False) would still show the initial progress bar without disable=True
         disable=not use_tqdm,
     )
     if tqdm_kwargs:
         _tqdm_kwargs.update(tqdm_kwargs)
-    with optional_context_manager(use_tqdm, tqdm(**_tqdm_kwargs)) as progress_bar, torch.no_grad():
+    with optional_context_manager(use_tqdm, tqdm(**_tqdm_kwargs)) as progress_bar, torch.inference_mode():
         # batch-wise processing
         for batch in batches:
             batch_size = batch.shape[0]
             relation_filter = None
             for column in (0, 2):
                 relation_filter = _evaluate_batch(
-                    batch=batch,
+                    batch=batch,  # TODO fix typing
                     model=model,
                     column=column,
                     filtered_evaluators=filtered_evaluators,
@@ -662,7 +751,7 @@ def _evaluate_batch(
         The relation filter, which can be re-used for the same batch.
     """
     if column not in {0, 2}:
-        raise ValueError(f'column must be either 0 or 2, but is column={column}')
+        raise ValueError(f"column must be either 0 or 2, but is column={column}")
 
     # Predict scores once
     if column == 2:  # tail scores
@@ -680,8 +769,10 @@ def _evaluate_batch(
     if filtering_necessary or positive_masks_required:
         # Needs all positive triples
         if all_pos_triples is None:
-            raise ValueError('If filtering_necessary of positive_masks_required is True, all_pos_triples has to be '
-                             'provided, but is None.')
+            raise ValueError(
+                "If filtering_necessary of positive_masks_required is True, all_pos_triples has to be "
+                "provided, but is None."
+            )
 
         # Create filter
         positive_filter, relation_filter = create_sparse_positive_filter_(
