@@ -2,7 +2,7 @@
 
 """An implementation of the extension to ERMLP."""
 
-from typing import Any, ClassVar, Mapping, Optional, Type
+from typing import Any, ClassVar, Mapping, Type
 
 import torch
 from torch import nn
@@ -11,43 +11,47 @@ from torch.nn.init import uniform_
 from ..base import EntityRelationEmbeddingModel
 from ...constants import DEFAULT_DROPOUT_HPO_RANGE, DEFAULT_EMBEDDING_HPO_EMBEDDING_DIM_RANGE
 from ...losses import BCEAfterSigmoidLoss, Loss
-from ...nn import EmbeddingSpecification
-from ...regularizers import Regularizer
-from ...triples import TriplesFactory
-from ...typing import DeviceHint, Hint, Initializer
+from ...nn.emb import EmbeddingSpecification
+from ...typing import Hint, Initializer
 
 __all__ = [
-    'ERMLPE',
+    "ERMLPE",
 ]
 
 
 class ERMLPE(EntityRelationEmbeddingModel):
-    r"""An extension of ERMLP proposed by [sharifzadeh2019]_.
+    r"""An extension of :class:`pykeen.models.ERMLP` proposed by [sharifzadeh2019]_.
 
-    This model uses a neural network-based approach similar to ERMLP and with slight modifications.
-    In ERMLP, the model is:
+    This model uses a neural network-based approach similar to ER-MLP and with slight modifications.
+    In ER-MLP, the model is:
 
     .. math::
 
         f(h, r, t) = \textbf{w}^{T} g(\textbf{W} [\textbf{h}; \textbf{r}; \textbf{t}])
 
-    whereas in ERMPLE the model is:
+    whereas in ER-MPL (E) the model is:
 
     .. math::
 
         f(h, r, t) = \textbf{t}^{T} f(\textbf{W} (g(\textbf{W} [\textbf{h}; \textbf{r}]))
 
     including dropouts and batch-norms between each two hidden layers.
-    ConvE can be seen as a special case of ERMLPE that contains the unnecessary inductive bias of convolutional
-    filters. The aim of this model is to show that lifting this bias from ConvE (which simply leaves us with a
-    modified ERMLP model), not only reduces the number of parameters but also improves performance.
-
+    ConvE can be seen as a special case of ER-MLP (E )that contains the unnecessary inductive bias of convolutional
+    filters. The aim of this model is to show that lifting this bias from :class:`pykeen.models.ConvE` (which simply
+    leaves us with a modified ER-MLP model), not only reduces the number of parameters but also improves performance.
+    ---
+    name: ER-MLP (E)
+    citation:
+        author: Sharifzadeh
+        year: 2019
+        link: https://github.com/pykeen/pykeen
+        github: pykeen/pykeen
     """
 
     #: The default strategy for optimizing the model's hyper-parameters
     hpo_default: ClassVar[Mapping[str, Any]] = dict(
         embedding_dim=DEFAULT_EMBEDDING_HPO_EMBEDDING_DIM_RANGE,
-        hidden_dim=dict(type=int, low=5, high=9, scale='power_two'),
+        hidden_dim=dict(type=int, low=5, high=9, scale="power_two"),
         input_dropout=DEFAULT_DROPOUT_HPO_RANGE,
         hidden_dropout=DEFAULT_DROPOUT_HPO_RANGE,
     )
@@ -58,24 +62,16 @@ class ERMLPE(EntityRelationEmbeddingModel):
 
     def __init__(
         self,
-        triples_factory: TriplesFactory,
+        *,
         hidden_dim: int = 300,
         input_dropout: float = 0.2,
         hidden_dropout: float = 0.3,
         embedding_dim: int = 200,
-        loss: Optional[Loss] = None,
-        preferred_device: DeviceHint = None,
-        random_seed: Optional[int] = None,
-        regularizer: Optional[Regularizer] = None,
         entity_initializer: Hint[Initializer] = uniform_,
         relation_initializer: Hint[Initializer] = uniform_,
+        **kwargs,
     ) -> None:
         super().__init__(
-            triples_factory=triples_factory,
-            loss=loss,
-            preferred_device=preferred_device,
-            random_seed=random_seed,
-            regularizer=regularizer,
             entity_representations=EmbeddingSpecification(
                 embedding_dim=embedding_dim,
                 initializer=entity_initializer,
@@ -84,6 +80,7 @@ class ERMLPE(EntityRelationEmbeddingModel):
                 embedding_dim=embedding_dim,
                 initializer=relation_initializer,
             ),
+            **kwargs,
         )
         self.hidden_dim = hidden_dim
 
