@@ -5,20 +5,17 @@
 from typing import Any, ClassVar, Mapping, Optional
 
 import torch
-import torch.autograd
 
 from ..base import EntityRelationEmbeddingModel
 from ...constants import DEFAULT_EMBEDDING_HPO_EMBEDDING_DIM_RANGE
-from ...losses import Loss
-from ...nn import EmbeddingSpecification
+from ...moves import irfft, rfft
+from ...nn.emb import EmbeddingSpecification
 from ...nn.init import xavier_uniform_
-from ...regularizers import Regularizer
-from ...triples import TriplesFactory
-from ...typing import Constrainer, DeviceHint, Hint, Initializer
+from ...typing import Constrainer, Hint, Initializer
 from ...utils import clamp_norm
 
 __all__ = [
-    'HolE',
+    "HolE",
 ]
 
 
@@ -48,6 +45,12 @@ class HolE(EntityRelationEmbeddingModel):
        - `author's implementation of HolE <https://github.com/mnick/holographic-embeddings>`_
        - `scikit-kge implementation of HolE <https://github.com/mnick/scikit-kge>`_
        - OpenKE `implementation of HolE <https://github.com/thunlp/OpenKE/blob/OpenKE-PyTorch/models/TransE.py>`_
+    ---
+    citation:
+        author: Nickel
+        year: 2016
+        link: https://www.aaai.org/ocs/index.php/AAAI/AAAI16/paper/viewFile/12484/11828
+        github: mnick/holographic-embeddings
     """
 
     #: The default strategy for optimizing the model's hyper-parameters
@@ -56,28 +59,20 @@ class HolE(EntityRelationEmbeddingModel):
     )
 
     #: The default settings for the entity constrainer
-    entity_constrainer_default_kwargs = dict(maxnorm=1., p=2, dim=-1)
+    entity_constrainer_default_kwargs = dict(maxnorm=1.0, p=2, dim=-1)
 
     def __init__(
         self,
-        triples_factory: TriplesFactory,
+        *,
         embedding_dim: int = 200,
-        loss: Optional[Loss] = None,
-        preferred_device: DeviceHint = None,
-        random_seed: Optional[int] = None,
-        regularizer: Optional[Regularizer] = None,
         entity_initializer: Hint[Initializer] = xavier_uniform_,
         entity_constrainer: Hint[Constrainer] = clamp_norm,  # type: ignore
         entity_constrainer_kwargs: Optional[Mapping[str, Any]] = None,
         relation_initializer: Hint[Constrainer] = xavier_uniform_,
+        **kwargs,
     ) -> None:
         """Initialize the model."""
         super().__init__(
-            triples_factory=triples_factory,
-            loss=loss,
-            preferred_device=preferred_device,
-            random_seed=random_seed,
-            regularizer=regularizer,
             entity_representations=EmbeddingSpecification(
                 embedding_dim=embedding_dim,
                 # Initialisation, cf. https://github.com/mnick/scikit-kge/blob/master/skge/param.py#L18-L27
@@ -89,6 +84,7 @@ class HolE(EntityRelationEmbeddingModel):
                 embedding_dim=embedding_dim,
                 initializer=relation_initializer,
             ),
+            **kwargs,
         )
 
     @staticmethod
@@ -112,17 +108,21 @@ class HolE(EntityRelationEmbeddingModel):
             The scores.
         """
         # Circular correlation of entity embeddings
-        a_fft = torch.rfft(h, signal_ndim=1, onesided=True)
-        b_fft = torch.rfft(t, signal_ndim=1, onesided=True)
+        a_fft = rfft(h, dim=-1)
+        b_fft = rfft(t, dim=-1)
 
         # complex conjugate, a_fft.shape = (batch_size, num_entities, d', 2)
-        a_fft[:, :, :, 1] *= -1
+        # compatibility: new style fft returns complex tensor
+        if a_fft.ndimension() > 3:
+            a_fft[:, :, :, 1] *= -1
+        else:
+            a_fft = torch.conj(a_fft)
 
         # Hadamard product in frequency domain
         p_fft = a_fft * b_fft
 
         # inverse real FFT, shape: (batch_size, num_entities, d)
-        composite = torch.irfft(p_fft, signal_ndim=1, onesided=True, signal_sizes=(h.shape[-1],))
+        composite = irfft(p_fft, dim=-1, n=h.shape[-1])
 
         # inner product with relation embedding
         scores = torch.sum(r * composite, dim=-1, keepdim=False)
