@@ -2,22 +2,20 @@
 
 """Implementation of the RotatE model."""
 
-from typing import Any, ClassVar, Mapping, Optional
+from typing import Any, ClassVar, Mapping
 
 import torch
 import torch.autograd
+from torch import linalg
 
 from ..base import EntityRelationEmbeddingModel
-from ...losses import Loss
-from ...nn import EmbeddingSpecification
+from ...nn.emb import EmbeddingSpecification
 from ...nn.init import init_phases, xavier_uniform_
-from ...nn.norm import complex_normalize
-from ...regularizers import Regularizer
-from ...triples import TriplesFactory
-from ...typing import Constrainer, DeviceHint, Hint, Initializer
+from ...typing import Constrainer, Hint, Initializer
+from ...utils import complex_normalize
 
 __all__ = [
-    'RotatE',
+    "RotatE",
 ]
 
 
@@ -44,6 +42,12 @@ class RotatE(EntityRelationEmbeddingModel):
 
        - Authors' `implementation of RotatE
          <https://github.com/DeepGraphLearning/KnowledgeGraphEmbedding/blob/master/codes/model.py#L200-L228>`_
+    ---
+    citation:
+        author: Sun
+        year: 2019
+        link: https://arxiv.org/abs/1902.10197v1
+        github: DeepGraphLearning/KnowledgeGraphEmbedding
     """
 
     #: The default strategy for optimizing the model's hyper-parameters
@@ -53,22 +57,14 @@ class RotatE(EntityRelationEmbeddingModel):
 
     def __init__(
         self,
-        triples_factory: TriplesFactory,
+        *,
         embedding_dim: int = 200,
-        loss: Optional[Loss] = None,
-        preferred_device: DeviceHint = None,
-        random_seed: Optional[int] = None,
-        regularizer: Optional[Regularizer] = None,
         entity_initializer: Hint[Initializer] = xavier_uniform_,
         relation_initializer: Hint[Initializer] = init_phases,
         relation_constrainer: Hint[Constrainer] = complex_normalize,
+        **kwargs,
     ) -> None:
         super().__init__(
-            triples_factory=triples_factory,
-            loss=loss,
-            preferred_device=preferred_device,
-            random_seed=random_seed,
-            regularizer=regularizer,
             entity_representations=EmbeddingSpecification(
                 embedding_dim=embedding_dim,
                 initializer=entity_initializer,
@@ -80,6 +76,7 @@ class RotatE(EntityRelationEmbeddingModel):
                 constrainer=relation_constrainer,
                 dtype=torch.cfloat,
             ),
+            **kwargs,
         )
         self.real_embedding_dim = embedding_dim
 
@@ -121,7 +118,7 @@ class RotatE(EntityRelationEmbeddingModel):
         )
         # Workaround until https://github.com/pytorch/pytorch/issues/30704 is fixed
         diff = rot_h - t
-        scores = -torch.norm(diff.view(diff.shape[:-2] + (-1,)), dim=-1)
+        scores = -linalg.vector_norm(diff, dim=(-2, -1))
 
         return scores
 
