@@ -6,19 +6,17 @@ from typing import Any, ClassVar, Mapping, Optional
 
 import torch
 import torch.autograd
+from torch import linalg
 
 from ..base import EntityRelationEmbeddingModel
 from ...constants import DEFAULT_EMBEDDING_HPO_EMBEDDING_DIM_RANGE
-from ...losses import Loss
-from ...nn import Embedding, EmbeddingSpecification
+from ...nn.emb import Embedding, EmbeddingSpecification
 from ...nn.init import xavier_normal_, xavier_uniform_, xavier_uniform_norm_
-from ...regularizers import Regularizer
-from ...triples import TriplesFactory
-from ...typing import Constrainer, DeviceHint, Hint, Initializer
+from ...typing import Constrainer, Hint, Initializer
 from ...utils import clamp_norm
 
 __all__ = [
-    'TransD',
+    "TransD",
 ]
 
 
@@ -99,6 +97,11 @@ class TransD(EntityRelationEmbeddingModel):
     .. seealso::
 
        - OpenKE `implementation of TransD <https://github.com/thunlp/OpenKE/blob/master/models/TransD.py>`_
+    ---
+    citation:
+        author: Ji
+        year: 2015
+        link: http://www.aclweb.org/anthology/P15-1067
     """
 
     #: The default strategy for optimizing the model's hyper-parameters
@@ -107,48 +110,46 @@ class TransD(EntityRelationEmbeddingModel):
         relation_dim=DEFAULT_EMBEDDING_HPO_EMBEDDING_DIM_RANGE,
     )
 
+    #: Secondary embeddings for entities
+    entity_projections: Embedding
+    #: Secondary embeddings for relations
+    relation_projections: Embedding
+
     def __init__(
         self,
-        triples_factory: TriplesFactory,
+        *,
         embedding_dim: int = 50,
         relation_dim: int = 30,
-        loss: Optional[Loss] = None,
-        preferred_device: DeviceHint = None,
-        random_seed: Optional[int] = None,
-        regularizer: Optional[Regularizer] = None,
         entity_initializer: Hint[Initializer] = xavier_uniform_,
         relation_initializer: Hint[Initializer] = xavier_uniform_norm_,
         entity_constrainer: Hint[Constrainer] = clamp_norm,  # type: ignore
         relation_constrainer: Hint[Constrainer] = clamp_norm,  # type: ignore
+        **kwargs,
     ) -> None:
         super().__init__(
-            triples_factory=triples_factory,
-            loss=loss,
-            preferred_device=preferred_device,
-            random_seed=random_seed,
-            regularizer=regularizer,
             entity_representations=EmbeddingSpecification(
                 embedding_dim=embedding_dim,
                 initializer=entity_initializer,
                 constrainer=entity_constrainer,
-                constrainer_kwargs=dict(maxnorm=1., p=2, dim=-1),
+                constrainer_kwargs=dict(maxnorm=1.0, p=2, dim=-1),
             ),
             relation_representations=EmbeddingSpecification(
                 embedding_dim=relation_dim,
                 initializer=relation_initializer,
                 constrainer=relation_constrainer,
-                constrainer_kwargs=dict(maxnorm=1., p=2, dim=-1),
+                constrainer_kwargs=dict(maxnorm=1.0, p=2, dim=-1),
             ),
+            **kwargs,
         )
 
         self.entity_projections = Embedding.init_with_device(
-            num_embeddings=triples_factory.num_entities,
+            num_embeddings=self.num_entities,
             embedding_dim=embedding_dim,
             device=self.device,
             initializer=xavier_normal_,
         )
         self.relation_projections = Embedding.init_with_device(
-            num_embeddings=triples_factory.num_relations,
+            num_embeddings=self.num_relations,
             embedding_dim=relation_dim,
             device=self.device,
             initializer=xavier_normal_,
@@ -193,7 +194,7 @@ class TransD(EntityRelationEmbeddingModel):
         t_bot = _project_entity(e=t, e_p=t_p, r=r, r_p=r_p)
 
         # score = -||h_bot + r - t_bot||_2^2
-        return -torch.norm(h_bot + r - t_bot, dim=-1, p=2) ** 2
+        return -linalg.vector_norm(h_bot + r - t_bot, dim=-1, ord=2) ** 2
 
     def _score(
         self,
