@@ -11,92 +11,66 @@ and (batch_size, 1, 1, num_tails, ``*``), and return a score tensor of shape
 from __future__ import annotations
 
 import functools
-from dataclasses import dataclass
-from typing import Optional, Tuple, Union
+from typing import Optional, Sequence, Tuple, Union
 
 import numpy
 import torch
-import torch.fft
-from torch import nn
+from torch import broadcast_tensors, nn
 
-from .compute_kernel import _complex_native_complex
+from .compute_kernel import batched_complex, batched_dot
 from .sim import KG2E_SIMILARITIES
-from ..typing import GaussianDistribution
+from ..moves import irfft, rfft
+from ..typing import GaussianDistribution, Sign
 from ..utils import (
-    broadcast_cat, clamp_norm, estimate_cost_of_sequence, extended_einsum, is_cudnn_error, negative_norm,
-    negative_norm_of_sum, project_entity, tensor_product, tensor_sum, view_complex,
+    boxe_kg_arity_position_score,
+    broadcast_cat,
+    clamp_norm,
+    compute_box,
+    estimate_cost_of_sequence,
+    extended_einsum,
+    is_cudnn_error,
+    negative_norm,
+    negative_norm_of_sum,
+    project_entity,
+    tensor_product,
+    tensor_sum,
+    view_complex,
 )
 
 __all__ = [
-    'complex_interaction',
-    'conve_interaction',
-    'convkb_interaction',
-    'distmult_interaction',
-    'ermlp_interaction',
-    'ermlpe_interaction',
-    'hole_interaction',
-    'kg2e_interaction',
-    'ntn_interaction',
-    'proje_interaction',
-    'rescal_interaction',
-    'rotate_interaction',
-    'simple_interaction',
-    'structured_embedding_interaction',
-    'transd_interaction',
-    'transe_interaction',
-    'transh_interaction',
-    'transr_interaction',
-    'tucker_interaction',
-    'unstructured_model_interaction',
+    "auto_sf_interaction",
+    "boxe_interaction",
+    "complex_interaction",
+    "conve_interaction",
+    "convkb_interaction",
+    "cp_interaction",
+    "cross_e_interaction",
+    "dist_ma_interaction",
+    "distmult_interaction",
+    "ermlp_interaction",
+    "ermlpe_interaction",
+    "hole_interaction",
+    "kg2e_interaction",
+    "mure_interaction",
+    "ntn_interaction",
+    "pair_re_interaction",
+    "proje_interaction",
+    "rescal_interaction",
+    "rotate_interaction",
+    "simple_interaction",
+    "se_interaction",
+    "transd_interaction",
+    "transe_interaction",
+    "transf_interaction",
+    "transh_interaction",
+    "transr_interaction",
+    "transformer_interaction",
+    "triple_re_interaction",
+    "tucker_interaction",
+    "um_interaction",
 ]
 
 
-@dataclass
-class SizeInformation:
-    """Size information of generic score function."""
-
-    #: The batch size of the head representations.
-    bh: int
-
-    #: The number of head representations per batch
-    nh: int
-
-    #: The batch size of the relation representations.
-    br: int
-
-    #: The number of relation representations per batch
-    nr: int
-
-    #: The batch size of the tail representations.
-    bt: int
-
-    #: The number of tail representations per batch
-    nt: int
-
-    @property
-    def same(self) -> bool:
-        """Whether all representations have the same shape."""
-        return (
-            self.bh == self.br
-            and self.bh == self.bt
-            and self.nh == self.nr
-            and self.nh == self.nt
-        )
-
-    @classmethod
-    def extract(
-        cls,
-        h: torch.Tensor,
-        r: torch.Tensor,
-        t: torch.Tensor,
-    ) -> SizeInformation:
-        """Extract size information from tensors."""
-        bh, nh = h.shape[:2]
-        br, nr = r.shape[:2]
-        bt, nt = t.shape[:2]
-        return cls(bh=bh, nh=nh, br=br, nr=nr, bt=bt, nt=nt)
-
-
 def _extract_sizes(
     h: torch.Tensor,
     r: torch.Tensor,
@@ -132,10 +106,10 @@ def _add_cuda_warning(func):
             if not is_cudnn_error(e):
                 raise e
             raise RuntimeError(
-                '\nThis code crash might have been caused by a CUDA bug, see '
-                'https://github.com/allenai/allennlp/issues/2888, '
-                'which causes the code to crash during evaluation mode.\n'
-                'To avoid this error, the batch size has to be reduced.',
+                "\nThis code crash might have been caused by a CUDA bug, see "
+                "https://github.com/allenai/allennlp/issues/2888, "
+                "which causes the code to crash during evaluation mode.\n"
+                "To avoid this error, the batch size has to be reduced.",
             ) from e
 
     return wrapped
@@ -161,7 +135,7 @@ def complex_interaction(
     :return: shape: (batch_size, num_heads, num_relations, num_tails)
         The scores.
     """
-    return _complex_native_complex(h, r, t)
+    return batched_complex(h, r, t)
 
 
 @_add_cuda_warning
@@ -203,8 +177,10 @@ def conve_interaction(
     # repeat if necessary, and concat head and relation, batch_size', num_input_channels, 2*height, width
     # with batch_size' = batch_size * num_heads * num_relations
     x = broadcast_cat(
-        h.view(*h.shape[:-1], input_channels, embedding_height, embedding_width),
-        r.view(*r.shape[:-1], input_channels, embedding_height, embedding_width),
+        [
+            h.view(*h.shape[:-1], input_channels, embedding_height, embedding_width),
+            r.view(*r.shape[:-1], input_channels, embedding_height, embedding_width),
+        ],
         dim=-2,
     ).view(-1, input_channels, 2 * embedding_height, embedding_width)
 
@@ -275,8 +251,7 @@ def convkb_interaction(
     # -> conv_head, conv_rel, conv_tail shapes: (num_filters,)
     # reshape to (1, 1, 1, 1, f, 1)
     conv_head, conv_rel, conv_tail, conv_bias = [
-        c.view(1, 1, 1, 1, num_filters, 1)
-        for c in list(conv.weight[:, 0, 0, :].t()) + [conv.bias]
+        c.view(1, 1, 1, 1, num_filters, 1) for c in list(conv.weight[:, 0, 0, :].t()) + [conv.bias]
     ]
 
     # convolve -> output.shape: (*, embedding_dim, num_filters)
@@ -316,6 +291,29 @@ def distmult_interaction(
     return tensor_product(h, r, t).sum(dim=-1)
 
 
+def dist_ma_interaction(
+    h: torch.FloatTensor,
+    r: torch.FloatTensor,
+    t: torch.FloatTensor,
+) -> torch.FloatTensor:
+    r"""Evaluate the DistMA interaction function from [shi2019]_.
+
+    .. math ::
+        \langle h, r\rangle + \langle r, t\rangle + \langle h, t\rangle
+
+    :param h: shape: (batch_size, num_heads, 1, 1, dim)
+        The head representations.
+    :param r: shape: (batch_size, 1, num_relations, 1, dim)
+        The relation representations.
+    :param t: shape: (batch_size, 1, 1, num_tails, dim)
+        The tail representations.
+
+    :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        The scores.
+    """
+    return batched_dot(h, r) + batched_dot(r, t) + batched_dot(h, t)
+
+
 def ermlp_interaction(
     h: torch.FloatTensor,
     r: torch.FloatTensor,
@@ -342,13 +340,11 @@ def ermlp_interaction(
     :return: shape: (batch_size, num_heads, num_relations, num_tails)
         The scores.
     """
-    sizes = SizeInformation.extract(h, r, t)
-
     # same shape
-    if sizes.same:
-        return final(activation(
-            hidden(torch.cat([h, r, t], dim=-1).view(-1, 3 * h.shape[-1]))),
-        ).view(sizes.bh, sizes.nh, sizes.nr, sizes.nt)
+    if h.shape == r.shape and h.shape == t.shape:
+        return final(
+            activation(hidden(torch.cat([h, r, t], dim=-1).view(-1, 3 * h.shape[-1]))),
+        ).view(*h.shape[:-1])
 
     hidden_dim = hidden.weight.shape[0]
     # split, shape: (embedding_dim, hidden_dim)
@@ -381,7 +377,7 @@ def ermlpe_interaction(
         The scores.
     """
     # repeat if necessary, and concat head and relation, (batch_size, num_heads, num_relations, 1, 2 * embedding_dim)
-    x = broadcast_cat(h, r, dim=-1)
+    x = broadcast_cat([h, r], dim=-1)
 
     # Predict t embedding, shape: (b, h, r, 1, d)
     shape = x.shape
@@ -411,18 +407,8 @@ def hole_interaction(
     :return: shape: (batch_size, num_heads, num_relations, num_tails)
         The scores.
     """
-    # Circular correlation of entity embeddings
-    a_fft = torch.fft.rfft(h, dim=-1)
-    b_fft = torch.fft.rfft(t, dim=-1)
-
-    # complex conjugate
-    a_fft = torch.conj(a_fft)
-
-    # Hadamard product in frequency domain
-    p_fft = a_fft * b_fft
-
-    # inverse real FFT, shape: (b, h, 1, t, d)
-    composite = torch.fft.irfft(p_fft, n=h.shape[-1], dim=-1)
+    # composite: (b, h, 1, t, d)
+    composite = circular_correlation(h, t)
 
     # transpose composite: (b, h, 1, d, t)
     composite = composite.transpose(-2, -1)
@@ -431,6 +417,35 @@ def hole_interaction(
     return (r @ composite).squeeze(dim=-2)
 
 
+def circular_correlation(
+    a: torch.FloatTensor,
+    b: torch.FloatTensor,
+) -> torch.FloatTensor:
+    """
+    Compute the circular correlation between to vectors.
+
+    .. note ::
+        The implementation uses FFT.
+
+    :param a: shape: s_1
+        The tensor with the first vectors.
+    :param b:
+        The tensor with the second vectors.
+
+    :return:
+        The circular correlation between the vectors.
+    """
+    # Circular correlation of entity embeddings
+    a_fft = rfft(a, dim=-1)
+    b_fft = rfft(b, dim=-1)
+    # complex conjugate
+    a_fft = torch.conj(a_fft)
+    # Hadamard product in frequency domain
+    p_fft = a_fft * b_fft
+    # inverse real FFT
+    return irfft(p_fft, n=a.shape[-1], dim=-1)
+
+
 def kg2e_interaction(
     h_mean: torch.FloatTensor,
     h_var: torch.FloatTensor,
@@ -507,12 +522,14 @@ def ntn_interaction(
     :return: shape: (batch_size, num_heads, num_relations, num_tails)
         The scores.
     """
-    x = activation(tensor_sum(
-        extended_einsum("bhrtd,bhrtkde,bhrte->bhrtk", h, w, t),
-        (vh @ h.unsqueeze(dim=-1)).squeeze(dim=-1),
-        (vt @ t.unsqueeze(dim=-1)).squeeze(dim=-1),
-        b,
-    ))
+    x = activation(
+        tensor_sum(
+            extended_einsum("bhrtd,bhrtkde,bhrte->bhrtk", h, w, t),
+            (vh @ h.unsqueeze(dim=-1)).squeeze(dim=-1),
+            (vt @ t.unsqueeze(dim=-1)).squeeze(dim=-1),
+            b,
+        )
+    )
     u = u.transpose(-2, -1)
     return (x @ u).squeeze(dim=-1)
 
@@ -657,7 +674,7 @@ def simple_interaction(
     return scores
 
 
-def structured_embedding_interaction(
+def se_interaction(
     h: torch.FloatTensor,
     r_h: torch.FloatTensor,
     r_t: torch.FloatTensor,
@@ -679,7 +696,7 @@ def structured_embedding_interaction(
     :param t: shape: (batch_size, 1, 1, num_tails, dim)
         The tail representations.
     :param p:
-        The p for the norm. cf. torch.norm.
+        The p for the norm. cf. :func:`torch.linalg.vector_norm`.
     :param power_norm:
         Whether to return the powered norm.
 
@@ -693,6 +710,38 @@ def structured_embedding_interaction(
     )
 
 
+def toruse_interaction(
+    h: torch.FloatTensor,
+    r: torch.FloatTensor,
+    t: torch.FloatTensor,
+    p: Union[int, str] = 2,
+    power_norm: bool = False,
+) -> torch.FloatTensor:
+    """Evaluate the TorusE interaction function from [ebisu2018].
+
+    .. note ::
+        This only implements the two L_p norm based variants.
+
+    :param h: shape: (batch_size, num_heads, 1, 1, dim)
+        The head representations.
+    :param r: shape: (batch_size, 1, num_relations, 1, dim)
+        The relation representations.
+    :param t: shape: (batch_size, 1, 1, num_tails, dim)
+        The tail representations.
+    :param p:
+        The p for the norm.
+    :param power_norm:
+        Whether to return the powered norm.
+
+    :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        The scores.
+    """
+    d = tensor_sum(h, r, -t)
+    d = d - torch.floor(d)
+    d = torch.minimum(d, 1.0 - d)
+    return negative_norm(d, p=p, power_norm=power_norm)
+
+
 def transd_interaction(
     h: torch.FloatTensor,
     r: torch.FloatTensor,
@@ -765,6 +814,26 @@ def transe_interaction(
     return negative_norm_of_sum(h, r, -t, p=p, power_norm=power_norm)
 
 
+def transf_interaction(
+    h: torch.FloatTensor,
+    r: torch.FloatTensor,
+    t: torch.FloatTensor,
+) -> torch.FloatTensor:
+    """Evaluate the TransF interaction function.
+
+    :param h: shape: (batch_size, num_heads, 1, 1, dim)
+        The head representations.
+    :param r: shape: (batch_size, 1, num_relations, 1, dim)
+        The relation representations.
+    :param t: shape: (batch_size, 1, 1, num_tails, dim)
+        The tail representations.
+
+    :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        The scores.
+    """
+    return batched_dot(h + r, t) + batched_dot(h, t - r)
+
+
 def transh_interaction(
     h: torch.FloatTensor,
     w_r: torch.FloatTensor,
@@ -784,7 +853,7 @@ def transh_interaction(
     :param t: shape: (batch_size, 1, 1, num_tails, dim)
         The tail representations.
     :param p:
-        The p for the norm. cf. torch.norm.
+        The p for the norm. cf. :func:`torch.linalg.vector_norm`.
     :param power_norm:
         Whether to return $|x-y|_p^p$.
 
@@ -832,8 +901,8 @@ def transr_interaction(
         The scores.
     """
     # project to relation specific subspace and ensure constraints
-    h_bot = clamp_norm((h.unsqueeze(dim=-2) @ m_r), p=2, dim=-1, maxnorm=1.).squeeze(dim=-2)
-    t_bot = clamp_norm((t.unsqueeze(dim=-2) @ m_r), p=2, dim=-1, maxnorm=1.).squeeze(dim=-2)
+    h_bot = clamp_norm((h.unsqueeze(dim=-2) @ m_r), p=2, dim=-1, maxnorm=1.0).squeeze(dim=-2)
+    t_bot = clamp_norm((t.unsqueeze(dim=-2) @ m_r), p=2, dim=-1, maxnorm=1.0).squeeze(dim=-2)
     return negative_norm_of_sum(h_bot, r, -t_bot, p=p, power_norm=power_norm)
 
 
@@ -900,7 +969,8 @@ def tucker_interaction(
                     x=h,
                     batch_norm=bn_h,
                     output_dropout=do_h,
-                )),
+                ),
+            ),
             batch_norm=bn_hr,
             output_dropout=do_hr,
         ),
@@ -908,7 +978,55 @@ def tucker_interaction(
     )
 
 
-def unstructured_model_interaction(
+def mure_interaction(
+    h: torch.FloatTensor,
+    b_h: torch.FloatTensor,
+    r_vec: torch.FloatTensor,
+    r_mat: torch.FloatTensor,
+    t: torch.FloatTensor,
+    b_t: torch.FloatTensor,
+    p: Union[int, float, str] = 2,
+    power_norm: bool = False,
+) -> torch.FloatTensor:
+    r"""Evaluate the MuRE interaction function from [balazevic2019b]_.
+
+    .. math ::
+        -\|Rh + r - t\| + b_h + b_t
+
+    :param h: shape: (batch_size, num_heads, 1, 1, dim)
+        The head representations.
+    :param b_h: shape: (batch_size, num_heads, 1, 1)
+        The head entity bias.
+    :param r_vec: shape: (batch_size, 1, num_relations, 1, dim)
+        The relation vector.
+    :param r_mat: shape: (batch_size, 1, num_relations, 1, dim,)
+        The diagonal relation matrix.
+    :param t: shape: (batch_size, 1, 1, num_tails, dim)
+        The tail representations.
+    :param b_t: shape: (batch_size, 1, 1, num_tails)
+        The tail entity bias.
+    :param p:
+        The parameter p for selecting the norm, cf. :func:`torch.linalg.vector_norm`.
+    :param power_norm:
+        Whether to return the powered norm instead.
+
+    :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        The scores.
+    """
+    return (
+        negative_norm_of_sum(
+            h * r_mat,
+            r_vec,
+            -t,
+            p=p,
+            power_norm=power_norm,
+        )
+        + b_h
+        + b_t
+    )
+
+
+def um_interaction(
     h: torch.FloatTensor,
     t: torch.FloatTensor,
     p: int,
@@ -929,3 +1047,415 @@ def unstructured_model_interaction(
         The scores.
     """
     return negative_norm(h - t, p=p, power_norm=power_norm)
+
+
+def pair_re_interaction(
+    h: torch.FloatTensor,
+    t: torch.FloatTensor,
+    r_h: torch.FloatTensor,
+    r_t: torch.FloatTensor,
+    p: Union[int, str] = 2,
+    power_norm: bool = True,
+) -> torch.FloatTensor:
+    r"""Evaluate the PairRE interaction function.
+
+    .. math ::
+        -\|h \odot r_h - t \odot r_t \|
+
+    :param h: shape: (batch_size, num_heads, 1, 1, dim)
+        The head representations.
+    :param t: shape: (batch_size, 1, 1, num_tails, dim)
+        The tail representations.
+    :param r_h: shape: (batch_size, 1, num_relations, 1, dim)
+        The head part of the relation representations.
+    :param r_t: shape: (batch_size, 1, num_relations, 1, dim)
+        The tail part of the relation representations.
+    :param p:
+        The parameter p for selecting the norm.
+    :param power_norm:
+        Whether to return the powered norm instead.
+
+    :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        The scores.
+    """
+    return negative_norm_of_sum(
+        h * r_h,
+        -t * r_t,
+        p=p,
+        power_norm=power_norm,
+    )
+
+
+def _rotate_quaternion(qa: torch.FloatTensor, qb: torch.FloatTensor) -> torch.FloatTensor:
+    # Rotate (=Hamilton product in quaternion space).
+    return torch.cat(
+        [
+            qa[0] * qb[0] - qa[1] * qb[1] - qa[2] * qb[2] - qa[3] * qb[3],
+            qa[0] * qb[1] + qa[1] * qb[0] + qa[2] * qb[3] - qa[3] * qb[2],
+            qa[0] * qb[2] - qa[1] * qb[3] + qa[2] * qb[0] + qa[3] * qb[1],
+            qa[0] * qb[3] + qa[1] * qb[2] - qa[2] * qb[1] + qa[3] * qb[0],
+        ],
+        dim=-1,
+    )
+
+
+def _split_quaternion(x: torch.FloatTensor) -> torch.FloatTensor:
+    return torch.chunk(x, chunks=4, dim=-1)
+
+
+def quat_e_interaction(
+    h: torch.FloatTensor,
+    r: torch.FloatTensor,
+    t: torch.FloatTensor,
+):
+    """Evaluate the interaction function of QuatE for given embeddings.
+
+    The embeddings have to be in a broadcastable shape.
+
+    .. note ::
+        dim has to be divisible by 4.
+
+    :param h: shape: (batch_size, num_heads, 1, 1, dim)
+        The head representations.
+    :param r: shape: (batch_size, 1, num_relations, 1, dim)
+        The head representations.
+    :param t: shape: (batch_size, 1, 1, num_tails, dim)
+        The tail representations.
+
+    :return: shape: (...)
+        The scores.
+    """
+    return -(
+        # Rotation in quaternion space
+        _rotate_quaternion(
+            _split_quaternion(h),
+            _split_quaternion(r),
+        )
+        * t
+    ).sum(dim=-1)
+
+
+def cross_e_interaction(
+    h: torch.FloatTensor,
+    r: torch.FloatTensor,
+    c_r: torch.FloatTensor,
+    t: torch.FloatTensor,
+    bias: torch.FloatTensor,
+    activation: nn.Module,
+    dropout: Optional[nn.Dropout] = None,
+) -> torch.FloatTensor:
+    r"""
+    Evaluate the interaction function of CrossE for the given representations from [zhang2019b]_.
+
+    .. math ::
+        Dropout(Activation(c_r \odot h + c_r \odot h \odot r + b))^T t)
+
+    .. note ::
+        The representations have to be in a broadcastable shape.
+
+    .. note ::
+        The CrossE paper described an additional sigmoid activation as part of the interaction function. Since using a
+        log-likelihood loss can cause numerical problems (due to explicitly calling sigmoid before log), we do not
+        apply this in our implementation but rather opt for the numerically stable variant. However, the model itself
+        has an option ``predict_with_sigmoid``, which can be used to enforce application of sigmoid during inference.
+        This can also have an impact of rank-based evaluation, since limited numerical precision can lead to exactly
+        equal scores for multiple choices. The definition of a rank is not unambiguous in such case, and there exist
+        multiple competing variants how to break the ties. More information on this can be found in the documentation of
+        rank-based evaluation.
+
+    :param h: shape: (batch_size, num_heads, 1, 1, dim)
+        The head representations.
+    :param r: shape: (batch_size, 1, num_relations, 1, dim)
+        The relation representations.
+    :param c_r: shape: (batch_size, 1, num_relations, 1, dim)
+        The relation-specific interaction vector.
+    :param t: shape: (batch_size, 1, 1, num_tails, dim)
+        The tail representations.
+    :param bias: shape: (1, 1, 1, 1, dim)
+        The combination bias.
+    :param activation:
+        The combination activation. Should be :class:`torch.nn.Tanh` for consistency with the CrossE paper.
+    :param dropout:
+        Dropout applied after the combination.
+
+    :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        The scores.
+
+    .. seealso:: https://github.com/wencolani/CrossE
+    """
+    # head interaction
+    h = c_r * h
+    # relation interaction (notice that h has been updated)
+    r = h * r
+    # combination
+    x = activation(h + r + bias)
+    if dropout is not None:
+        x = dropout(x)
+    # similarity
+    return (x * t).sum(dim=-1)
+
+
+def boxe_interaction(
+    # head
+    h_pos: torch.FloatTensor,
+    h_bump: torch.FloatTensor,
+    # relation box: head
+    rh_base: torch.FloatTensor,
+    rh_delta: torch.FloatTensor,
+    rh_size: torch.FloatTensor,
+    # relation box: tail
+    rt_base: torch.FloatTensor,
+    rt_delta: torch.FloatTensor,
+    rt_size: torch.FloatTensor,
+    # tail
+    t_pos: torch.FloatTensor,
+    t_bump: torch.FloatTensor,
+    # power norm
+    tanh_map: bool = True,
+    p: int = 2,
+    power_norm: bool = False,
+) -> torch.FloatTensor:
+    """
+    Evalute the BoxE interaction function from [abboud2020]_.
+
+    Entities are described via position and bump. Relations are described as a pair of boxes, where each box is
+    parametrized as triple (base, delta, size), where # TODO
+
+    .. note ::
+        this interaction relies on Abboud's point-to-box distance
+        :func:`pykeen.utils.point_to_box_distance`.
+
+    :param h_pos: shape: (batch_size, num_heads, 1, 1, d)
+        the head entity position
+    :param h_bump: shape: (batch_size, num_heads, 1, 1, d)
+        the head entity bump
+
+    :param rh_base: shape: (batch_size, 1, num_relations, 1, d)
+        the relation-specific head box base position
+    :param rh_delta: shape: (batch_size, 1, num_relations, 1, d)
+        # the relation-specific head box base shape (normalized to have a volume of 1):
+    :param rh_size: shape: (batch_size, 1, num_relations, 1, 1)
+        the relation-specific head box size (a scalar)
+    :param rt_base: shape: (batch_size, 1, num_relations, 1, d)
+        the relation-specific tail box base position
+    :param rt_delta: shape: (batch_size, 1, num_relations, 1, d)
+        # the relation-specific tail box base shape (normalized to have a volume of 1):
+    :param rt_size: shape: (batch_size, 1, num_relations, 1, d)
+        the relation-specific tail box size
+
+    :param t_pos: shape: (batch_size, 1, 1, num_tails, d)
+        the tail entity position
+    :param t_bump: shape: (batch_size, 1, 1, num_tails, d)
+        the tail entity bump
+
+    :param tanh_map:
+        whether to apply the tanh mapping
+    :param p:
+        the order of the norm to apply
+    :param power_norm:
+        whether to use the p-th power of the p-norm instead
+
+    :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        The scores.
+    """
+    return sum(
+        boxe_kg_arity_position_score(
+            entity_pos=entity_pos,
+            other_entity_bump=other_entity_pos,
+            relation_box=compute_box(base=base, delta=delta, size=size),
+            tanh_map=tanh_map,
+            p=p,
+            power_norm=power_norm,
+        )
+        for entity_pos, other_entity_pos, base, delta, size in (
+            (h_pos, t_bump, rh_base, rh_delta, rh_size),
+            (t_pos, h_bump, rt_base, rt_delta, rt_size),
+        )
+    )
+
+
+def cp_interaction(
+    h: torch.FloatTensor,
+    r: torch.FloatTensor,
+    t: torch.FloatTensor,
+) -> torch.FloatTensor:
+    """Evaluate the Canonical Tensor Decomposition interaction function.
+
+    :param h: shape: (batch_size, num_heads, 1, 1, rank, dim)
+        The head representations.
+    :param r: shape: (batch_size, 1, num_relations, 1, rank, dim)
+        The relation representations.
+    :param t: shape: (batch_size, 1, 1, num_tails, rank, dim)
+        The tail representations.
+
+    :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        The scores.
+    """
+    return extended_einsum(
+        "bhrtkd,bhrtkd,bhrtkd->bhrt",
+        h,
+        r,
+        t,
+    )
+
+
+def triple_re_interaction(
+    # head
+    h: torch.FloatTensor,
+    # relation
+    r_head: torch.FloatTensor,
+    r_mid: torch.FloatTensor,
+    r_tail: torch.FloatTensor,
+    # tail
+    t: torch.FloatTensor,
+    # version 2: relation factor offset
+    u: Optional[float] = None,
+    # extension: negative (power) norm
+    p: int = 2,
+    power_norm: bool = False,
+) -> torch.FloatTensor:
+    r"""Evaluate the TripleRE interaction function.
+
+    .. math ::
+        score(h, (r_h, r, r_t), t) = h * (r_h + u) - t * (r_t + u) + r
+
+    .. note ::
+
+        For equivalence to the paper version, `h` and `t` should be normalized to unit
+        Euclidean length, and `p` and `power_norm` be kept at their default values.
+
+    :param h: shape: (batch_size, num_heads, 1, 1, rank, dim)
+        The head representations.
+    :param r_head: shape: (batch_size, 1, num_relations, 1, rank, dim)
+        The relation-specific head multiplicator representations.
+    :param r_mid: shape: (batch_size, 1, num_relations, 1, rank, dim)
+        The relation representations.
+    :param r_tail: shape: (batch_size, 1, num_relations, 1, rank, dim)
+        The relation-specific tail multiplicator representations.
+    :param t: shape: (batch_size, 1, 1, num_tails, rank, dim)
+        The tail representations.
+    :param u:
+        the relation factor offset. If u is not None or 0, this corresponds to TripleREv2.
+    :param p:
+        The p for the norm. cf. :func:`torch.linalg.vector_norm`.
+    :param power_norm:
+        Whether to return the powered norm.
+
+    :return: shape: (batch_size, num_heads, num_relations, num_tails)
+        The scores.
+    """
+    # note: normalization should be done from the representations
+    # cf. https://github.com/LongYu-360/TripleRE-Add-NodePiece/blob/994216dcb1d718318384368dd0135477f852c6a4/TripleRE%2BNodepiece/ogb_wikikg2/model.py#L317-L328  # noqa: E501
+    # version 2
+    if u is not None:
+        # r_head = r_head + u * torch.ones_like(r_head)
+        # r_tail = r_tail + u * torch.ones_like(r_tail)
+        r_head = r_head + u
+        r_tail = r_tail + u
+
+    return negative_norm_of_sum(
+        h * r_head,
+        -t * r_tail,
+        r_mid,
+        p=p,
+        power_norm=power_norm,
+    )
+
+
+def auto_sf_interaction(
+    h: Sequence[torch.FloatTensor],
+    r: Sequence[torch.FloatTensor],
+    t: Sequence[torch.FloatTensor],
+    coefficients: Sequence[Tuple[int, int, int, Sign]],
+) -> torch.FloatTensor:
+    r"""Evaluate an AutoSF-style interaction function as described by [zhang2020]_.
+
+    This interaction function is a parametrized way to express bi-linear models
+    with block structure. It divides the entity and relation representations into blocks,
+    and expresses the interaction as a sequence of 4-tuples $(i_h, i_r, i_t, s)$,
+    where $i_h, i_r, i_t$ index a _block_ of the head, relation, or tail representation,
+    and $s \in {-1, 1}$ is the sign.
+
+    The interaction function is then given as
+
+    .. math::
+        \sum_{(i_h, i_r, i_t, s) \in \mathcal{C}} s \cdot \langle h[i_h], r[i_r], t[i_t] \rangle
+
+    where $\langle \cdot, \cdot, \cdot \rangle$ denotes the tri-linear dot product.
+
+    This parametrization allows to express several well-known interaction functions, e.g.
+
+    - :class:`pykeen.models.DistMult`: one block, $\mathcal{C} = \{(0, 0, 0, 1)\}$
+    - :class:`pykeen.models.ComplEx`: two blocks,
+      $\mathcal{C} = \{(0, 0, 0, 1), (0, 1, 1, 1), (1, 0, 1, -1), (1, 0, 1, 1)\}$
+    - :class:`pykeen.models.SimplE`: two blocks: $\mathcal{C} = \{(0, 0, 1, 1), (1, 1, 0, 1)\}$
+
+    :param h: each shape: (batch_size, num_heads, 1, 1, rank, dim)
+        The list of head representations.
+    :param r: each shape: (batch_size, 1, num_relations, 1, rank, dim)
+        The list of relation representations.
+    :param t: each shape: (batch_size, 1, 1, num_tails, rank, dim)
+        The list of tail representations.
+    :param coefficients:
+        the coefficients, in order:
+
+        1. head_representation_index,
+        2. relation_representation_index,
+        3. tail_representation_index,
+        4. sign
+    """
+    return sum(sign * (h[hi] * r[ri] * t[ti]).sum(dim=-1) for hi, ri, ti, sign in coefficients)
+
+
+def transformer_interaction(
+    h: torch.FloatTensor,
+    r: torch.FloatTensor,
+    t: torch.FloatTensor,
+    transformer: nn.TransformerEncoder,
+    position_embeddings: torch.FloatTensor,
+    final: nn.Module,
+) -> torch.FloatTensor:
+    r"""Evaluate the Transformer interaction function, as described in [galkin2020]_..
+
+    .. math ::
+
+        \textit{score}(h, r, t) =
+            \textit{Linear}(\textit{SumPooling}(\textit{Transformer}([h + pe[0]; r + pe[1]])))^T t
+
+    :param h: shape: (batch_size, num_heads, 1, 1, dim)
+        The head representations.
+    :param r: shape: (batch_size, 1, num_relations, 1, dim)
+        The relation representations.
+    :param t: shape: (batch_size, 1, 1, num_tails, dim)
+        The tail representations.
+    :param transformer:
+        the transformer encoder
+    :param position_embeddings: shape: (2, dim)
+        the positional embeddings, one for head and one for relation
+    :param final:
+        the final (linear) transformation
+    """
+    # stack h & r (+ broadcast) => shape: (2, batch_size', num_heads, num_relations, 1, *dims)
+    x = torch.stack(broadcast_tensors(h, r), dim=0)
+
+    # remember shape for output, but reshape for transformer
+    hr_shape = x.shape
+    x = x.view(2, -1, *hr_shape[5:])
+
+    # get position embeddings, shape: (seq_len, dim)
+    # Now we are position-dependent w.r.t qualifier pairs.
+    x = x + position_embeddings.unsqueeze(dim=1)
+
+    # seq_length, batch_size, dim
+    x = transformer(src=x)
+
+    # Pool output
+    x = x.sum(dim=0)
+
+    # output shape: (batch_size, dim)
+    x = final(x)
+
+    # reshape
+    x = x.view(*hr_shape[1:5], x.shape[-1])
+
+    return (x @ t.transpose(-1, -2)).squeeze(dim=-2)
