@@ -2,15 +2,13 @@
 
 """Negative sampling algorithm based on the work of [wang2014]_."""
 
-from typing import Optional, Tuple
-
 import torch
 
 from .negative_sampler import NegativeSampler
-from ..triples import TriplesFactory
+from ..triples import CoreTriplesFactory
 
 __all__ = [
-    'BernoulliNegativeSampler',
+    "BernoulliNegativeSampler",
 ]
 
 
@@ -36,48 +34,45 @@ class BernoulliNegativeSampler(NegativeSampler):
     actual positive triples $(h,r,t) \in \mathcal{K}$ will be removed.
     """
 
-    #: The default strategy for optimizing the negative sampler's hyper-parameters
-    hpo_default = dict(
-        num_negs_per_pos=dict(type=int, low=1, high=100, q=10),
-    )
-
     def __init__(
         self,
-        triples_factory: TriplesFactory,
-        num_negs_per_pos: Optional[int] = None,
-        filtered: bool = False,
+        *,
+        triples_factory: CoreTriplesFactory,
+        **kwargs,
     ) -> None:
-        super().__init__(
-            triples_factory=triples_factory,
-            num_negs_per_pos=num_negs_per_pos,
-            filtered=filtered,
-        )
+        """Initialize the bernoulli negative sampler with the given entities.
+
+        :param triples_factory:
+            The factory holding the positive training triples
+        :param kwargs:
+            Additional keyword based arguments passed to :class:`pykeen.sampling.NegativeSampler`.
+        """
+        super().__init__(triples_factory=triples_factory, **kwargs)
         # Preprocessing: Compute corruption probabilities
-        triples = self.triples_factory.mapped_triples
+        triples = triples_factory.mapped_triples
         head_rel_uniq, tail_count = torch.unique(triples[:, :2], return_counts=True, dim=0)
         rel_tail_uniq, head_count = torch.unique(triples[:, 1:], return_counts=True, dim=0)
 
         self.corrupt_head_probability = torch.empty(
-            self.triples_factory.num_relations,
+            triples_factory.num_relations,
             device=triples_factory.mapped_triples.device,
         )
 
-        for r in range(self.triples_factory.num_relations):
+        for r in range(triples_factory.num_relations):
             # compute tph, i.e. the average number of tail entities per head
-            mask = (head_rel_uniq[:, 1] == r)
+            mask = head_rel_uniq[:, 1] == r
             tph = tail_count[mask].float().mean()
 
             # compute hpt, i.e. the average number of head entities per tail
-            mask = (rel_tail_uniq[:, 0] == r)
+            mask = rel_tail_uniq[:, 0] == r
             hpt = head_count[mask].float().mean()
 
             # Set parameter for Bernoulli distribution
             self.corrupt_head_probability[r] = tph / (tph + hpt)
 
-    def sample(self, positive_batch: torch.LongTensor) -> Tuple[torch.LongTensor, Optional[torch.Tensor]]:
-        """Sample a negative batched based on the bern approach."""
+    def corrupt_batch(self, positive_batch: torch.LongTensor) -> torch.LongTensor:  # noqa: D102
         if self.num_negs_per_pos > 1:
-            positive_batch = positive_batch.repeat(self.num_negs_per_pos, 1)
+            positive_batch = positive_batch.repeat_interleave(repeats=self.num_negs_per_pos, dim=0)
 
         # Bind number of negatives to sample
         num_negs = positive_batch.shape[0]
@@ -94,10 +89,13 @@ class BernoulliNegativeSampler(NegativeSampler):
         # Tails are corrupted if heads are not corrupted
         tail_mask = ~head_mask
 
-        # Randomly sample corruption. See below for explanation of
-        # why this is on a range of [0, num_entities - 1]
+        # We at least make sure to not replace the triples by the original value
+        # See below for explanation of why this is on a range of [0, num_entities - 1]
+        index_max = self.num_entities - 1
+
+        # Randomly sample corruption.
         negative_entities = torch.randint(
-            self.triples_factory.num_entities - 1,
+            index_max,
             size=(num_negs,),
             device=positive_batch.device,
         )
@@ -108,15 +106,10 @@ class BernoulliNegativeSampler(NegativeSampler):
         # Replace tails
         negative_batch[tail_mask, 2] = negative_entities[tail_mask]
 
-        # If filtering is activated, all negative triples that are positive in the training dataset will be removed
-        if self.filtered:
-            negative_batch, batch_filter = self.filter_negative_triples(negative_batch=negative_batch)
-        else:
-            # To make sure we don't replace the head by the original value
-            # we shift all values greater or equal than the original value by one up
-            # for that reason we choose the random value from [0, num_entities -1]
-            negative_batch[head_mask, 0] += (negative_batch[head_mask, 0] >= positive_batch[head_mask, 0]).long()
-            negative_batch[tail_mask, 2] += (negative_batch[tail_mask, 2] >= positive_batch[tail_mask, 2]).long()
-            batch_filter = None
+        # To make sure we don't replace the head by the original value
+        # we shift all values greater or equal than the original value by one up
+        # for that reason we choose the random value from [0, num_entities -1]
+        negative_batch[head_mask, 0] += (negative_batch[head_mask, 0] >= positive_batch[head_mask, 0]).long()
+        negative_batch[tail_mask, 2] += (negative_batch[tail_mask, 2] >= positive_batch[tail_mask, 2]).long()
 
-        return negative_batch, batch_filter
+        return negative_batch.view(-1, self.num_negs_per_pos, 3)
