@@ -5,9 +5,25 @@
 import dataclasses
 import itertools
 import logging
-import os
+import pathlib
 import re
-from typing import Any, Callable, Collection, Dict, List, Mapping, Optional, Sequence, Set, TextIO, Union
+from abc import abstractmethod
+from typing import (
+    Any,
+    Callable,
+    Collection,
+    Dict,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Set,
+    TextIO,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 
 import numpy as np
 import pandas as pd
@@ -15,22 +31,26 @@ import torch
 
 from .instances import Instances, LCWAInstances, SLCWAInstances
 from .splitting import split
-from .utils import load_triples
+from .utils import TRIPLES_DF_COLUMNS, get_entities, get_relations, load_triples, tensor_to_df, triple_tensor_to_set
 from ..typing import EntityMapping, LabeledTriples, MappedTriples, RelationMapping, TorchRandomHint
-from ..utils import compact_mapping, format_relative_comparison, invert_mapping, torch_is_in_1d
+from ..utils import compact_mapping, format_relative_comparison, invert_mapping
 
 __all__ = [
-    'CoreTriplesFactory',
-    'TriplesFactory',
-    'create_entity_mapping',
-    'create_relation_mapping',
-    'INVERSE_SUFFIX',
+    "CoreTriplesFactory",
+    "TriplesFactory",
+    "create_entity_mapping",
+    "create_relation_mapping",
+    "INVERSE_SUFFIX",
+    "cat_triples",
+    "splits_steps",
+    "splits_similarity",
+    "RelationInverter",
+    "relation_inverter",
 ]
 
 logger = logging.getLogger(__name__)
 
-INVERSE_SUFFIX = '_inverse'
-TRIPLES_DF_COLUMNS = ('head_id', 'head_label', 'relation_id', 'relation_label', 'tail_id', 'tail_label')
+INVERSE_SUFFIX = "_inverse"
 
 
 def create_entity_mapping(triples: LabeledTriples) -> EntityMapping:
@@ -43,10 +63,7 @@ def create_entity_mapping(triples: LabeledTriples) -> EntityMapping:
     # Sorting ensures consistent results when the triples are permuted
     entity_labels = sorted(set(heads).union(tails))
     # Create mapping
-    return {
-        str(label): i
-        for (i, label) in enumerate(entity_labels)
-    }
+    return {str(label): i for (i, label) in enumerate(entity_labels)}
 
 
 def create_relation_mapping(relations: set) -> RelationMapping:
@@ -57,13 +74,10 @@ def create_relation_mapping(relations: set) -> RelationMapping:
     # Sorting ensures consistent results when the triples are permuted
     relation_labels = sorted(
         set(relations),
-        key=lambda x: (re.sub(f'{INVERSE_SUFFIX}$', '', x), x.endswith(f'{INVERSE_SUFFIX}')),
+        key=lambda x: (re.sub(f"{INVERSE_SUFFIX}$", "", x), x.endswith(f"{INVERSE_SUFFIX}")),
     )
     # Create mapping
-    return {
-        str(label): i
-        for (i, label) in enumerate(relation_labels)
-    }
+    return {str(label): i for (i, label) in enumerate(relation_labels)}
 
 
 def _map_triples_elements_to_ids(
@@ -73,7 +87,7 @@ def _map_triples_elements_to_ids(
 ) -> MappedTriples:
     """Map entities and relations to pre-defined ids."""
     if triples.size == 0:
-        logger.warning('Provided empty triples to map.')
+        logger.warning("Provided empty triples to map.")
         return torch.empty(0, 3, dtype=torch.long)
 
     # When triples that don't exist are trying to be mapped, they get the id "-1"
@@ -96,7 +110,7 @@ def _map_triples_elements_to_ids(
             f"You're trying to map triples with {num_no_head + num_no_tail} entities and {num_no_relation} relations"
             f" that are not in the training set. These triples will be excluded from the mapping.",
         )
-        non_mappable_triples = (head_filter | relation_filter | tail_filter)
+        non_mappable_triples = head_filter | relation_filter | tail_filter
         head_column = head_column[~non_mappable_triples, None]
         relation_column = relation_column[~non_mappable_triples, None]
         tail_column = tail_column[~non_mappable_triples, None]
@@ -106,7 +120,7 @@ def _map_triples_elements_to_ids(
 
     triples_of_ids = np.concatenate([head_column, relation_column, tail_column], axis=1)
 
-    triples_of_ids = np.array(triples_of_ids, dtype=np.long)
+    triples_of_ids = np.array(triples_of_ids, dtype=np.int64)
     # Note: Unique changes the order of the triples
     # Note: Using unique means implicit balancing of training samples
     unique_mapped_triples = np.unique(ar=triples_of_ids, axis=0)
@@ -124,10 +138,10 @@ def _get_triple_mask(
     triples = triples[:, columns]
     if isinstance(columns, int):
         columns = [columns]
-    mask = torch_is_in_1d(
-        query_tensor=triples,
-        test_tensor=ids,
-        max_id=max_id,
+    mask = torch.isin(
+        elements=triples,
+        test_elements=torch.as_tensor(list(ids), dtype=torch.long),
+        assume_unique=False,
         invert=invert,
     )
     if len(columns) > 1:
@@ -140,10 +154,57 @@ def _ensure_ids(
     label_to_id: Mapping[str, int],
 ) -> Collection[int]:
     """Convert labels to IDs."""
-    return [
-        label_to_id[l_or_i] if isinstance(l_or_i, str) else l_or_i
-        for l_or_i in labels_or_ids
-    ]
+    return [label_to_id[l_or_i] if isinstance(l_or_i, str) else l_or_i for l_or_i in labels_or_ids]
+
+
+RelationID = TypeVar("RelationID", int, torch.LongTensor)
+
+
+class RelationInverter:
+    """An interface for inverse-relation ID mapping."""
+
+    # TODO: method is_inverse?
+
+    @abstractmethod
+    def get_inverse_id(self, relation_id: RelationID) -> RelationID:
+        """Get the inverse ID for a given relation."""
+        # TODO: inverse of inverse?
+        raise NotImplementedError
+
+    @abstractmethod
+    def _map(self, batch: torch.LongTensor, index: int = 1) -> torch.LongTensor:
+        raise NotImplementedError
+
+    @abstractmethod
+    def invert_(self, batch: torch.LongTensor, index: int = 1) -> torch.LongTensor:
+        """Invert relations in a batch (in-place)."""
+        raise NotImplementedError
+
+    def map(self, batch: torch.LongTensor, index: int = 1, invert: bool = False) -> torch.LongTensor:
+        """Map relations of batch, optionally also inverting them."""
+        batch = self._map(batch=batch, index=index)
+        return self.invert_(batch=batch, index=index) if invert else batch
+
+
+class DefaultRelationInverter(RelationInverter):
+    """Maps normal relations to even IDs, and the corresponding inverse to the next odd ID."""
+
+    def get_inverse_id(self, relation_id: RelationID) -> RelationID:  # noqa: D102
+        return relation_id + 1
+
+    def _map(self, batch: torch.LongTensor, index: int = 1, invert: bool = False) -> torch.LongTensor:  # noqa: D102
+        batch = batch.clone()
+        batch[:, index] *= 2
+        return batch
+
+    def invert_(self, batch: torch.LongTensor, index: int = 1) -> torch.LongTensor:  # noqa: D102
+        # The number of relations stored in the triples factory includes the number of inverse relations
+        # Id of inverse relation: relation + 1
+        batch[:, index] += 1
+        return batch
+
+
+relation_inverter = DefaultRelationInverter()
 
 
 @dataclasses.dataclass
@@ -157,16 +218,16 @@ class Labeling:
     id_to_label: Mapping[int, str] = dataclasses.field(init=False)
 
     #: A vectorized version of entity_label_to_id; initialized automatically
-    _vectorized_mapper: Callable[..., np.ndarray] = dataclasses.field(init=False)
+    _vectorized_mapper: Callable[..., np.ndarray] = dataclasses.field(init=False, compare=False)
 
     #: A vectorized version of entity_id_to_label; initialized automatically
-    _vectorized_labeler: Callable[..., np.ndarray] = dataclasses.field(init=False)
+    _vectorized_labeler: Callable[..., np.ndarray] = dataclasses.field(init=False, compare=False)
 
     def __post_init__(self):
         """Precompute inverse mappings."""
         self.id_to_label = invert_mapping(mapping=self.label_to_id)
-        self._vectorized_mapper = np.vectorize(self.label_to_id.get)
-        self._vectorized_labeler = np.vectorize(self.id_to_label.get)
+        self._vectorized_mapper = np.vectorize(self.label_to_id.get, otypes=[int])
+        self._vectorized_labeler = np.vectorize(self.id_to_label.get, otypes=[str])
 
     def label(
         self,
@@ -184,6 +245,56 @@ class Labeling:
         return self._vectorized_labeler(ids, (unknown_label,))
 
 
+def restrict_triples(
+    mapped_triples: MappedTriples,
+    entities: Optional[Collection[int]] = None,
+    relations: Optional[Collection[int]] = None,
+    invert_entity_selection: bool = False,
+    invert_relation_selection: bool = False,
+) -> MappedTriples:
+    """Select a subset of triples based on the given entity and relation ID selection.
+
+    :param mapped_triples:
+        The ID-based triples.
+    :param entities:
+        The entity IDs of interest. If None, defaults to all entities.
+    :param relations:
+        The relation IDs of interest. If None, defaults to all relations.
+    :param invert_entity_selection:
+        Whether to invert the entity selection, i.e. select those triples without the provided entities.
+    :param invert_relation_selection:
+        Whether to invert the relation selection, i.e. select those triples without the provided relations.
+    :return:
+        A tensor of triples containing the entities and relations of interest.
+    """
+    keep_mask = None
+
+    # Filter for entities
+    if entities is not None:
+        keep_mask = _get_triple_mask(
+            ids=entities,
+            triples=mapped_triples,
+            columns=(0, 2),  # head and entity need to fulfil the requirement
+            invert=invert_entity_selection,
+        )
+
+    # Filter for relations
+    if relations is not None:
+        relation_mask = _get_triple_mask(
+            ids=relations,
+            triples=mapped_triples,
+            columns=1,
+            invert=invert_relation_selection,
+        )
+        keep_mask = relation_mask if keep_mask is None else keep_mask & relation_mask
+
+    # No filter
+    if keep_mask is None:
+        return mapped_triples
+
+    return mapped_triples[keep_mask]
+
+
 @dataclasses.dataclass
 class CoreTriplesFactory:
     """Create instances from ID-based triples."""
@@ -193,6 +304,8 @@ class CoreTriplesFactory:
         mapped_triples: MappedTriples,
         num_entities: int,
         num_relations: int,
+        entity_ids: Collection[int],
+        relation_ids: Collection[int],
         create_inverse_triples: bool = False,
         metadata: Optional[Mapping[str, Any]] = None,
     ):
@@ -214,6 +327,8 @@ class CoreTriplesFactory:
         self.mapped_triples = mapped_triples
         self._num_entities = num_entities
         self._num_relations = num_relations
+        self.entity_ids = entity_ids
+        self.relation_ids = relation_ids
         self.create_inverse_triples = create_inverse_triples
         if metadata is None:
             metadata = dict()
@@ -225,6 +340,8 @@ class CoreTriplesFactory:
         mapped_triples: MappedTriples,
         num_entities: Optional[int] = None,
         num_relations: Optional[int] = None,
+        entity_ids: Collection[int] = None,
+        relation_ids: Collection[int] = None,
         create_inverse_triples: bool = False,
         metadata: Optional[Mapping[str, Any]] = None,
     ) -> "CoreTriplesFactory":
@@ -249,10 +366,16 @@ class CoreTriplesFactory:
             num_entities = mapped_triples[:, [0, 2]].max().item() + 1
         if num_relations is None:
             num_relations = mapped_triples[:, 1].max().item() + 1
+        if entity_ids is None:
+            entity_ids = get_entities(mapped_triples)
+        if relation_ids is None:
+            relation_ids = get_relations(mapped_triples)
         return CoreTriplesFactory(
             mapped_triples=mapped_triples,
             num_entities=num_entities,
             num_relations=num_relations,
+            entity_ids=entity_ids,
+            relation_ids=relation_ids,
             create_inverse_triples=create_inverse_triples,
             metadata=metadata,
         )
@@ -279,30 +402,19 @@ class CoreTriplesFactory:
         """The number of triples."""
         return self.mapped_triples.shape[0]
 
-    def get_entity_ids(self) -> Collection[int]:
-        """Get the set of entity identifiers."""
-        raise NotImplementedError  # TODO @mberr should these be pre-cached on __init__?
-
-    def get_relation_ids(self) -> Collection[int]:
-        """Get the set of relation identifiers."""
-        raise NotImplementedError
-
     def extra_repr(self) -> str:
         """Extra representation string."""
         d = [
-            ('num_entities', self.num_entities),
-            ('num_relations', self.num_relations),
-            ('num_triples', self.num_triples),
-            ('inverse_triples', self.create_inverse_triples),
+            ("num_entities", self.num_entities),
+            ("num_relations", self.num_relations),
+            ("num_triples", self.num_triples),
+            ("inverse_triples", self.create_inverse_triples),
         ]
         d.extend(sorted(self.metadata.items()))  # type: ignore
-        return ', '.join(
-            f'{k}="{v}"' if isinstance(v, str) else f'{k}={v}'
-            for k, v in d
-        )
+        return ", ".join(f'{k}="{v}"' if isinstance(v, (str, pathlib.Path)) else f"{k}={v}" for k, v in d)
 
     def __repr__(self):  # noqa: D105
-        return f'{self.__class__.__name__}({self.extra_repr()})'
+        return f"{self.__class__.__name__}({self.extra_repr()})"
 
     def with_labels(
         self,
@@ -330,12 +442,8 @@ class CoreTriplesFactory:
     def get_inverse_relation_id(self, relation: int) -> int:
         """Get the inverse relation identifier for the given relation."""
         if not self.create_inverse_triples:
-            raise ValueError('Can not get inverse triple, they have not been created.')
-        return self._get_inverse_relation_id(relation)
-
-    @staticmethod
-    def _get_inverse_relation_id(relation_id: Union[int, torch.LongTensor]) -> Union[int, torch.LongTensor]:
-        return relation_id + 1
+            raise ValueError("Can not get inverse triple, they have not been created.")
+        return relation_inverter.get_inverse_id(relation_id=relation)
 
     def _add_inverse_triples_if_necessary(self, mapped_triples: MappedTriples) -> MappedTriples:
         """Add inverse triples if they shall be created."""
@@ -343,22 +451,27 @@ class CoreTriplesFactory:
             return mapped_triples
 
         logger.info("Creating inverse triples.")
-        h, r, t = mapped_triples.t()
-        r = 2 * r
-        return torch.cat([
-            torch.stack([h, r, t], dim=-1),
-            torch.stack([t, self._get_inverse_relation_id(r), h], dim=-1),
-        ])
+        return torch.cat(
+            [
+                relation_inverter.map(batch=mapped_triples),
+                relation_inverter.map(batch=mapped_triples, invert=True).flip(1),
+            ]
+        )
 
     def create_slcwa_instances(self) -> Instances:
         """Create sLCWA instances for this factory's triples."""
-        return SLCWAInstances(mapped_triples=self._add_inverse_triples_if_necessary(mapped_triples=self.mapped_triples))
+        return self._create_instances(SLCWAInstances)
 
-    def create_lcwa_instances(self, use_tqdm: Optional[bool] = None) -> Instances:
+    def create_lcwa_instances(self, use_tqdm: Optional[bool] = None, target: Optional[int] = None) -> Instances:
         """Create LCWA instances for this factory's triples."""
-        return LCWAInstances.from_triples(
+        return self._create_instances(LCWAInstances, target=target)
+
+    def _create_instances(self, instances_cls: Type[Instances], **kwargs) -> Instances:
+        return instances_cls.from_triples(
             mapped_triples=self._add_inverse_triples_if_necessary(mapped_triples=self.mapped_triples),
             num_entities=self.num_entities,
+            num_relations=self.num_relations,
+            **kwargs,
         )
 
     def get_most_frequent_relations(self, n: Union[int, float]) -> Set[int]:
@@ -367,12 +480,12 @@ class CoreTriplesFactory:
         :param n:
             Either the (integer) number of top relations to keep or the (float) percentage of top relationships to keep.
         """
-        logger.info(f'applying cutoff of {n} to {self}')
+        logger.info(f"applying cutoff of {n} to {self}")
         if isinstance(n, float):
             assert 0 < n < 1
             n = int(self.num_relations * n)
         elif not isinstance(n, int):
-            raise TypeError('n must be either an integer or a float')
+            raise TypeError("n must be either an integer or a float")
 
         uniq, counts = self.mapped_triples[:, 1].unique(return_counts=True)
         top_counts, top_ids = counts.topk(k=n, largest=True)
@@ -410,6 +523,8 @@ class CoreTriplesFactory:
             mapped_triples=mapped_triples,
             num_entities=self.num_entities,
             num_relations=self.real_num_relations,
+            entity_ids=self.entity_ids,
+            relation_ids=self.relation_ids,
             create_inverse_triples=create_inverse_triples,
             metadata={
                 **(extra_metadata or {}),
@@ -424,7 +539,7 @@ class CoreTriplesFactory:
         random_state: TorchRandomHint = None,
         randomize_cleanup: bool = False,
         method: Optional[str] = None,
-    ) -> List['CoreTriplesFactory']:
+    ) -> List["CoreTriplesFactory"]:
         """Split a triples factory into a train/test.
 
         :param ratios:
@@ -467,28 +582,30 @@ class CoreTriplesFactory:
                 # do not explicitly create inverse triples for testing; this is handled by the evaluation code
                 create_inverse_triples=None if i == 0 else False,
             )
-            for i, triples in enumerate(split(
-                mapped_triples=self.mapped_triples,
-                ratios=ratios,
-                random_state=random_state,
-                randomize_cleanup=randomize_cleanup,
-                method=method,
-            ))
+            for i, triples in enumerate(
+                split(
+                    mapped_triples=self.mapped_triples,
+                    ratios=ratios,
+                    random_state=random_state,
+                    randomize_cleanup=randomize_cleanup,
+                    method=method,
+                )
+            )
         ]
 
-    def get_mask_for_entities(
-        self,
-        entities: Union[Collection[int]],
-        invert: bool = False,
-    ) -> torch.BoolTensor:
-        """Get a boolean mask for triples with the given entities."""
-        return _get_triple_mask(
-            ids=entities,
-            triples=self.mapped_triples,
-            columns=(0, 2),  # head and entity need to fulfil the requirement
-            invert=invert,
-            max_id=self.num_entities,
-        )
+    def entities_to_ids(self, entities: Union[Collection[int], Collection[str]]) -> Collection[int]:
+        """Normalize entities to IDs."""
+        for e in entities:
+            if not isinstance(e, int):
+                raise ValueError(f"{self.__class__.__name__} cannot convert entity IDs from {type(e)} to int.")
+        return cast(Collection[int], entities)
+
+    def relations_to_ids(self, relations: Union[Collection[int], Collection[str]]) -> Collection[int]:
+        """Normalize relations to IDs."""
+        for e in relations:
+            if not isinstance(e, int):
+                raise ValueError(f"{self.__class__.__name__} cannot convert relation IDs from {type(e)} to int.")
+        return cast(Collection[int], relations)
 
     def get_mask_for_relations(
         self,
@@ -519,40 +636,15 @@ class CoreTriplesFactory:
         :return:
             A dataframe with n rows, and 6 + len(kwargs) columns.
         """
-        # Input validation
-        additional_columns = set(kwargs.keys())
-        forbidden = additional_columns.intersection(TRIPLES_DF_COLUMNS)
-        if len(forbidden) > 0:
-            raise ValueError(
-                f'The key-words for additional arguments must not be in {TRIPLES_DF_COLUMNS}, but {forbidden} were '
-                f'used.',
-            )
-
-        # convert to numpy
-        tensor = tensor.cpu().numpy()
-        data = dict(zip(['head_id', 'relation_id', 'tail_id'], tensor.T))
-
-        # Additional columns
-        for key, values in kwargs.items():
-            # convert PyTorch tensors to numpy
-            if isinstance(values, torch.Tensor):
-                values = values.cpu().numpy()
-            data[key] = values
-
-        # convert to dataframe
-        rv = pd.DataFrame(data=data)
-
-        # Re-order columns
-        columns = list(TRIPLES_DF_COLUMNS[::2]) + sorted(set(rv.columns).difference(TRIPLES_DF_COLUMNS))
-        return rv.loc[:, columns]
+        return tensor_to_df(tensor=tensor, **kwargs)
 
     def new_with_restriction(
         self,
-        entities: Union[None, Collection[int]] = None,
-        relations: Union[None, Collection[int]] = None,
+        entities: Union[None, Collection[int], Collection[str]] = None,
+        relations: Union[None, Collection[int], Collection[str]] = None,
         invert_entity_selection: bool = False,
         invert_relation_selection: bool = False,
-    ) -> 'CoreTriplesFactory':
+    ) -> "CoreTriplesFactory":
         """Make a new triples factory only keeping the given entities and relations, but keeping the ID mapping.
 
         :param entities:
@@ -568,35 +660,83 @@ class CoreTriplesFactory:
             A new triples factory, which has only a subset of the triples containing the entities and relations of
             interest. The label-to-ID mapping is *not* modified.
         """
-        keep_mask = None
-
+        # prepare metadata
         extra_metadata = {}
-        # Filter for entities
         if entities is not None:
-            extra_metadata['entity_restriction'] = entities
-            keep_mask = self.get_mask_for_entities(entities=entities, invert=invert_entity_selection)
-            remaining_entities = self.num_entities - len(entities) if invert_entity_selection else len(entities)
+            extra_metadata["entity_restriction"] = entities
+            entities = self.entities_to_ids(entities=entities)
+            remaining_entities = (self.num_entities - len(entities)) if invert_entity_selection else len(entities)
             logger.info(f"keeping {format_relative_comparison(remaining_entities, self.num_entities)} entities.")
-
-        # Filter for relations
         if relations is not None:
-            extra_metadata['relation_restriction'] = relations
-            relation_mask = self.get_mask_for_relations(relations=relations, invert=invert_relation_selection)
-            remaining_relations = self.num_relations - len(relations) if invert_entity_selection else len(relations)
+            extra_metadata["relation_restriction"] = relations
+            relations = self.relations_to_ids(relations=relations)
+            remaining_relations = (self.num_relations - len(relations)) if invert_relation_selection else len(relations)
             logger.info(f"keeping {format_relative_comparison(remaining_relations, self.num_relations)} relations.")
-            keep_mask = relation_mask if keep_mask is None else keep_mask & relation_mask
 
-        # No filtering happened
-        if keep_mask is None:
+        # Delegate to function
+        mapped_triples = restrict_triples(
+            mapped_triples=self.mapped_triples,
+            entities=entities,
+            relations=relations,
+            invert_entity_selection=invert_entity_selection,
+            invert_relation_selection=invert_relation_selection,
+        )
+
+        # restrict triples can only remove triples; thus, if the new size equals the old one, nothing has changed
+        if mapped_triples.shape[0] == self.num_triples:
             return self
 
-        num_triples = keep_mask.sum()
-        logger.info(f"keeping {format_relative_comparison(num_triples, self.num_triples)} triples.")
+        logger.info(f"keeping {format_relative_comparison(mapped_triples.shape[0], self.num_triples)} triples.")
+
         return self.clone_and_exchange_triples(
-            mapped_triples=self.mapped_triples[keep_mask],
+            mapped_triples=mapped_triples,
             extra_metadata=extra_metadata,
         )
 
+    @classmethod
+    def from_path_binary(
+        cls,
+        path: Union[str, pathlib.Path, TextIO],
+    ) -> "CoreTriplesFactory":  # noqa: D102
+        """
+        Load triples factory from a binary file.
+
+        :param path:
+            The path, pointing to an existing PyTorch .pt file.
+
+        :return:
+            The loaded triples factory.
+        """
+        path = normalize_path(path)
+        logger.info(f"Loading from {path.as_uri()}")
+        data = torch.load(path)
+        return cls(**data)
+
+    def to_path_binary(
+        self,
+        path: Union[str, pathlib.Path, TextIO],
+    ) -> None:
+        """
+        Save triples factory to path in (PyTorch's .pt) binary format.
+
+        :param path:
+            The path to store the triples factory to.
+        """
+        path = normalize_path(path)
+        torch.save(self._get_binary_state(), path)
+        logger.info(f"Stored {self} to {path.as_uri()}")
+
+    def _get_binary_state(self):
+        return dict(
+            mapped_triples=self.mapped_triples,
+            num_entities=self.num_entities,
+            num_relations=self.num_relations,
+            entity_ids=self.entity_ids,
+            relation_ids=self.relation_ids,
+            create_inverse_triples=self.create_inverse_triples,
+            metadata=self.metadata,
+        )
+
 
 class TriplesFactory(CoreTriplesFactory):
     """Create instances given the path to triples."""
@@ -627,6 +767,8 @@ class TriplesFactory(CoreTriplesFactory):
             mapped_triples=mapped_triples,
             num_entities=len(entity_to_id),
             num_relations=len(relation_to_id),
+            entity_ids=sorted(entity_to_id.values()),
+            relation_ids=sorted(relation_to_id.values()),
             create_inverse_triples=create_inverse_triples,
             metadata=metadata,
         )
@@ -643,7 +785,7 @@ class TriplesFactory(CoreTriplesFactory):
         compact_id: bool = True,
         filter_out_candidate_inverse_relations: bool = True,
         metadata: Optional[Dict[str, Any]] = None,
-    ) -> 'TriplesFactory':
+    ) -> "TriplesFactory":
         """
         Create a new triples factory from label-based triples.
 
@@ -673,14 +815,12 @@ class TriplesFactory(CoreTriplesFactory):
             suspected_to_be_inverse_relations = {r for r in unique_relations if r.endswith(INVERSE_SUFFIX)}
             if len(suspected_to_be_inverse_relations) > 0:
                 logger.warning(
-                    f'Some triples already have the inverse relation suffix {INVERSE_SUFFIX}. '
-                    f'Re-creating inverse triples to ensure consistency. You may disable this behaviour by passing '
-                    f'filter_out_candidate_inverse_relations=False',
+                    f"Some triples already have the inverse relation suffix {INVERSE_SUFFIX}. "
+                    f"Re-creating inverse triples to ensure consistency. You may disable this behaviour by passing "
+                    f"filter_out_candidate_inverse_relations=False",
                 )
                 relation_ids_to_remove = [
-                    i
-                    for i, r in enumerate(unique_relations.tolist())
-                    if r in suspected_to_be_inverse_relations
+                    i for i, r in enumerate(unique_relations.tolist()) if r in suspected_to_be_inverse_relations
                 ]
                 mask = np.isin(element=inverse, test_elements=relation_ids_to_remove, invert=True)
                 logger.info(f"keeping {mask.sum() / mask.shape[0]} triples.")
@@ -716,14 +856,14 @@ class TriplesFactory(CoreTriplesFactory):
     @classmethod
     def from_path(
         cls,
-        path: Union[str, TextIO],
+        path: Union[str, pathlib.Path, TextIO],
         create_inverse_triples: bool = False,
         entity_to_id: Optional[EntityMapping] = None,
         relation_to_id: Optional[RelationMapping] = None,
         compact_id: bool = True,
         metadata: Optional[Dict[str, Any]] = None,
         load_triples_kwargs: Optional[Mapping[str, Any]] = None,
-    ) -> 'TriplesFactory':
+    ) -> "TriplesFactory":
         """
         Create a new triples factory from triples stored in a file.
 
@@ -747,12 +887,7 @@ class TriplesFactory(CoreTriplesFactory):
         :return:
             A new triples factory.
         """
-        if isinstance(path, str):
-            path = os.path.abspath(path)
-        elif isinstance(path, TextIO):
-            path = os.path.abspath(path.name)
-        else:
-            raise TypeError(f'path is invalid type: {type(path)}')
+        path = normalize_path(path)
 
         # TODO: Check if lazy evaluation would make sense
         triples = load_triples(path, **(load_triples_kwargs or {}))
@@ -764,11 +899,32 @@ class TriplesFactory(CoreTriplesFactory):
             relation_to_id=relation_to_id,
             compact_id=compact_id,
             metadata={
-                'path': path,
+                "path": path,
                 **(metadata or {}),
             },
         )
 
+    def to_core_triples_factory(self) -> CoreTriplesFactory:
+        """Return this factory as a core factory."""
+        return CoreTriplesFactory(
+            mapped_triples=self.mapped_triples,
+            num_entities=self.num_entities,
+            num_relations=self.num_relations,
+            entity_ids=self.entity_ids,
+            relation_ids=self.relation_ids,
+            create_inverse_triples=self.create_inverse_triples,
+            metadata=self.metadata,
+        )
+
+    def _get_binary_state(self):
+        return dict(
+            mapped_triples=self.mapped_triples,
+            entity_to_id=self.entity_to_id,
+            relation_to_id=self.relation_to_id,
+            create_inverse_triples=self.create_inverse_triples,
+            metadata=self.metadata,
+        )
+
     def clone_and_exchange_triples(
         self,
         mapped_triples: MappedTriples,
@@ -809,14 +965,6 @@ class TriplesFactory(CoreTriplesFactory):
         """Return the mapping from relations IDs to labels."""
         return self.relation_labeling.id_to_label
 
-    def get_entity_ids(self) -> Collection[int]:
-        """Get the set of entity identifiers."""
-        return self.entity_to_id.values()
-
-    def get_relation_ids(self) -> Collection[int]:
-        """Get the set of relation identifiers."""
-        return self.relation_to_id.values()
-
     @property
     def triples(self) -> np.ndarray:  # noqa: D401
         """The labeled triples, a 3-column matrix where each row are the head label, relation label, then tail label."""
@@ -851,35 +999,25 @@ class TriplesFactory(CoreTriplesFactory):
             return np.empty(shape=(0, 3), dtype=str)
         if unknown_relation_label is None:
             unknown_relation_label = unknown_entity_label
-        return np.stack([
-            labeling.label(ids=column, unknown_label=unknown_label)
-            for (labeling, unknown_label), column in zip(
-                [
-                    (self.entity_labeling, unknown_entity_label),
-                    (self.relation_labeling, unknown_relation_label),
-                    (self.entity_labeling, unknown_entity_label),
-                ],
-                triples.t().numpy(),
-            )
-        ], axis=1)
+        return np.stack(
+            [
+                labeling.label(ids=column, unknown_label=unknown_label)
+                for (labeling, unknown_label), column in zip(
+                    [
+                        (self.entity_labeling, unknown_entity_label),
+                        (self.relation_labeling, unknown_relation_label),
+                        (self.entity_labeling, unknown_entity_label),
+                    ],
+                    triples.t().numpy(),
+                )
+            ],
+            axis=1,
+        )
 
-    def entities_to_ids(self, entities: Union[Collection[int], Collection[str]]) -> Collection[int]:
-        """Normalize entities to IDs."""
+    def entities_to_ids(self, entities: Union[Collection[int], Collection[str]]) -> Collection[int]:  # noqa: D102
         return _ensure_ids(labels_or_ids=entities, label_to_id=self.entity_labeling.label_to_id)
 
-    def get_mask_for_entities(
-        self,
-        entities: Union[Collection[int], Collection[str]],
-        invert: bool = False,
-    ) -> torch.BoolTensor:
-        """Get a boolean mask for triples with the given entities."""
-        return super().get_mask_for_entities(entities=self.entities_to_ids(entities=entities))
-
-    def relations_to_ids(
-        self,
-        relations: Union[Collection[int], Collection[str]],
-    ) -> Collection[int]:
-        """Normalize relations to IDs."""
+    def relations_to_ids(self, relations: Union[Collection[int], Collection[str]]) -> Collection[int]:  # noqa: D102
         return _ensure_ids(labels_or_ids=relations, label_to_id=self.relation_labeling.label_to_id)
 
     def get_mask_for_relations(
@@ -929,22 +1067,30 @@ class TriplesFactory(CoreTriplesFactory):
             from word_cloud.word_cloud_generator import WordCloud
         except ImportError:
             logger.warning(
-                'Could not import module `word_cloud`. '
-                'Try installing it with `pip install git+https://github.com/kavgan/word_cloud.git`',
+                "Could not import module `word_cloud`. "
+                "Try installing it with `pip install git+https://github.com/kavgan/word_cloud.git`",
             )
             return
 
         # pre-filter to keep only topk
         uniq, counts = ids.view(-1).unique(return_counts=True)
+
+        # if top is larger than the number of available options
+        top = min(top, uniq.numel())
         top_counts, top_ids = counts.topk(k=top, largest=True)
 
         # generate text
-        text = list(itertools.chain(*(
-            itertools.repeat(id_to_label[e_id], count)
-            for e_id, count in zip(top_ids.tolist(), top_counts.tolist())
-        )))
+        text = list(
+            itertools.chain(
+                *(
+                    itertools.repeat(id_to_label[e_id], count)
+                    for e_id, count in zip(top_ids.tolist(), top_counts.tolist())
+                )
+            )
+        )
 
         from IPython.core.display import HTML
+
         word_cloud = WordCloud()
         return HTML(word_cloud.get_embed_code(text=text, topn=top))
 
@@ -963,8 +1109,8 @@ class TriplesFactory(CoreTriplesFactory):
             tail=self.entity_labeling,
         ).items():
             assert labeling is not None
-            data[f'{column}_label'] = labeling.label(
-                ids=data[f'{column}_id'],
+            data[f"{column}_label"] = labeling.label(
+                ids=data[f"{column}_id"],
                 unknown_label=("[unknown_" + column + "]").upper(),
             )
 
@@ -978,16 +1124,72 @@ class TriplesFactory(CoreTriplesFactory):
         relations: Union[None, Collection[int], Collection[str]] = None,
         invert_entity_selection: bool = False,
         invert_relation_selection: bool = False,
-    ) -> 'TriplesFactory':  # noqa: D102
+    ) -> "TriplesFactory":  # noqa: D102
         if entities is None and relations is None:
             return self
         if entities is not None:
             entities = self.entities_to_ids(entities=entities)
         if relations is not None:
             relations = self.relations_to_ids(relations=relations)
-        return super().new_with_restriction(
-            entities=entities,
-            relations=relations,
-            invert_entity_selection=invert_entity_selection,
-            invert_relation_selection=invert_relation_selection,
-        ).with_labels(entity_to_id=self.entity_to_id, relation_to_id=self.relation_to_id)
+        return (
+            super()
+            .new_with_restriction(
+                entities=entities,
+                relations=relations,
+                invert_entity_selection=invert_entity_selection,
+                invert_relation_selection=invert_relation_selection,
+            )
+            .with_labels(entity_to_id=self.entity_to_id, relation_to_id=self.relation_to_id)
+        )
+
+    def map_triples(self, triples: LabeledTriples) -> MappedTriples:
+        """Convert label-based triples to ID-based triples."""
+        return _map_triples_elements_to_ids(
+            triples=triples,
+            entity_to_id=self.entity_to_id,
+            relation_to_id=self.relation_to_id,
+        )
+
+
+def cat_triples(*triples_factories: CoreTriplesFactory) -> MappedTriples:
+    """Concatenate several triples factories."""
+    return torch.cat([factory.mapped_triples for factory in triples_factories], dim=0)
+
+
+def splits_steps(a: Sequence[CoreTriplesFactory], b: Sequence[CoreTriplesFactory]) -> int:
+    """Compute the number of moves to go from the first sequence of triples factories to the second.
+
+    :return: The number of triples present in the training sets in both
+    """
+    if len(a) != len(b):
+        raise ValueError("Must have same number of triples factories")
+
+    train_1 = triple_tensor_to_set(a[0].mapped_triples)
+    train_2 = triple_tensor_to_set(b[0].mapped_triples)
+
+    # FIXME currently the implementation does not consider the non-training (i.e., second-last entries)
+    #  for the number of steps. Consider more interesting way to discuss splits w/ valid
+
+    return len(train_1.symmetric_difference(train_2))
+
+
+def splits_similarity(a: Sequence[CoreTriplesFactory], b: Sequence[CoreTriplesFactory]) -> float:
+    """Compute the similarity between two datasets' splits.
+
+    :return: The number of triples present in the training sets in both
+    """
+    steps = splits_steps(a, b)
+    n = sum(tf.num_triples for tf in a)
+    return 1 - steps / n
+
+
+def normalize_path(path: Union[str, pathlib.Path, TextIO]) -> pathlib.Path:
+    """Normalize path."""
+    if isinstance(path, TextIO):
+        return pathlib.Path(path.name).resolve()
+    elif isinstance(path, str):
+        return pathlib.Path(path).resolve()
+    elif isinstance(path, pathlib.Path):
+        return path.resolve()
+    else:
+        raise TypeError(f"path is invalid type: {type(path)}")
