@@ -5,142 +5,35 @@
 import dataclasses
 import logging
 import unittest
-from typing import Any, ClassVar, Dict, Mapping, Optional, Tuple, Type
+from operator import attrgetter
+from typing import Dict, Optional
 
+import numpy
 import torch
 
 from pykeen.datasets import Nations
 from pykeen.evaluation import Evaluator, MetricResults, RankBasedEvaluator, RankBasedMetricResults
+from pykeen.evaluation.classification_evaluator import ClassificationEvaluator, ClassificationMetricResults
 from pykeen.evaluation.evaluator import create_dense_positive_mask_, create_sparse_positive_filter_, filter_scores_
-from pykeen.evaluation.rank_based_evaluator import RANK_TYPES, SIDES, compute_rank_from_scores
-from pykeen.evaluation.sklearn import SklearnEvaluator, SklearnMetricResults
-from pykeen.models import Model, TransE
-from pykeen.triples import TriplesFactory
+from pykeen.evaluation.rank_based_evaluator import (
+    RANK_EXPECTED_REALISTIC,
+    RANK_OPTIMISTIC,
+    RANK_PESSIMISTIC,
+    RANK_REALISTIC,
+    RANK_TYPES,
+    SIDE_BOTH,
+    SIDES,
+    compute_rank_from_scores,
+    resolve_metric_name,
+)
+from pykeen.models import FixedModel
 from pykeen.typing import MappedTriples
-from tests.mocks import MockModel
+from tests import cases
 
 logger = logging.getLogger(__name__)
 
 
-class _AbstractEvaluatorTests:
-    """A test case for quickly defining common tests for evaluators models."""
-
-    # The triples factory and model
-    factory: TriplesFactory
-    model: Model
-
-    #: The evaluator to be tested
-    evaluator_cls: ClassVar[Type[Evaluator]]
-    evaluator_kwargs: ClassVar[Optional[Mapping[str, Any]]] = None
-
-    # Settings
-    batch_size: int
-    embedding_dim: int
-
-    #: The evaluator instantiation
-    evaluator: Evaluator
-
-    def setUp(self) -> None:
-        """Set up the test case."""
-        # Settings
-        self.batch_size = 8
-        self.embedding_dim = 7
-
-        # Initialize evaluator
-        self.evaluator = self.evaluator_cls(**(self.evaluator_kwargs or {}))
-
-        # Use small test dataset
-        self.factory = Nations().training
-
-        # Use small model (untrained)
-        self.model = TransE(triples_factory=self.factory, embedding_dim=self.embedding_dim)
-
-    def _get_input(
-        self,
-        inverse: bool = False,
-    ) -> Tuple[torch.LongTensor, torch.FloatTensor, Optional[torch.BoolTensor]]:
-        # Get batch
-        hrt_batch = self.factory.mapped_triples[:self.batch_size].to(self.model.device)
-
-        # Compute scores
-        if inverse:
-            scores = self.model.score_h(rt_batch=hrt_batch[:, 1:])
-        else:
-            scores = self.model.score_t(hr_batch=hrt_batch[:, :2])
-
-        # Compute mask only if required
-        if self.evaluator.requires_positive_mask:
-            # TODO: Re-use filtering code
-            triples = self.factory.mapped_triples.to(self.model.device)
-            if inverse:
-                sel_col, start_col = 0, 1
-            else:
-                sel_col, start_col = 2, 0
-            stop_col = start_col + 2
-
-            # shape: (batch_size, num_triples)
-            triple_mask = (triples[None, :, start_col:stop_col] == hrt_batch[:, None, start_col:stop_col]).all(dim=-1)
-            batch_indices, triple_indices = triple_mask.nonzero(as_tuple=True)
-            entity_indices = triples[triple_indices, sel_col]
-
-            # shape: (batch_size, num_entities)
-            mask = torch.zeros_like(scores, dtype=torch.bool)
-            mask[batch_indices, entity_indices] = True
-        else:
-            mask = None
-
-        return hrt_batch, scores, mask
-
-    def test_process_tail_scores_(self) -> None:
-        """Test the evaluator's ``process_tail_scores_()`` function."""
-        hrt_batch, scores, mask = self._get_input()
-        true_scores = scores[torch.arange(0, hrt_batch.shape[0]), hrt_batch[:, 2]][:, None]
-        self.evaluator.process_tail_scores_(
-            hrt_batch=hrt_batch,
-            true_scores=true_scores,
-            scores=scores,
-            dense_positive_mask=mask,
-        )
-
-    def test_process_head_scores_(self) -> None:
-        """Test the evaluator's ``process_head_scores_()`` function."""
-        hrt_batch, scores, mask = self._get_input(inverse=True)
-        true_scores = scores[torch.arange(0, hrt_batch.shape[0]), hrt_batch[:, 0]][:, None]
-        self.evaluator.process_head_scores_(
-            hrt_batch=hrt_batch,
-            true_scores=true_scores,
-            scores=scores,
-            dense_positive_mask=mask,
-        )
-
-    def test_finalize(self) -> None:
-        # Process one batch
-        hrt_batch, scores, mask = self._get_input()
-        true_scores = scores[torch.arange(0, hrt_batch.shape[0]), hrt_batch[:, 2]][:, None]
-        self.evaluator.process_tail_scores_(
-            hrt_batch=hrt_batch,
-            true_scores=true_scores,
-            scores=scores,
-            dense_positive_mask=mask,
-        )
-
-        result = self.evaluator.finalize()
-        assert isinstance(result, MetricResults)
-
-        self._validate_result(
-            result=result,
-            data={'batch': hrt_batch, 'scores': scores, 'mask': mask},
-        )
-
-    def _validate_result(
-        self,
-        result: MetricResults,
-        data: Dict[str, torch.Tensor],
-    ):
-        logger.warning(f'{self.__class__.__name__} did not overwrite _validate_result.')
-
-
-class RankBasedEvaluatorTests(_AbstractEvaluatorTests, unittest.TestCase):
+class RankBasedEvaluatorTests(cases.EvaluatorTestCase):
     """unittest for the RankBasedEvaluator."""
 
     evaluator_cls = RankBasedEvaluator
@@ -152,20 +45,26 @@ class RankBasedEvaluatorTests(_AbstractEvaluatorTests, unittest.TestCase):
     ):
         # Check for correct class
         assert isinstance(result, RankBasedMetricResults)
+        result: RankBasedMetricResults
 
         # Check value ranges
-        for side, all_type_mr in result.mean_rank.items():
+        # check mean rank (MR)
+        for side, all_type_mr in result.arithmetic_mean_rank.items():
             assert side in SIDES
             for rank_type, mr in all_type_mr.items():
                 assert rank_type in RANK_TYPES
                 assert isinstance(mr, float)
                 assert 1 <= mr <= self.factory.num_entities
-        for side, all_type_mrr in result.mean_reciprocal_rank.items():
+
+        # check mean reciprocal rank (MRR)
+        for side, all_type_mrr in result.inverse_harmonic_mean_rank.items():
             assert side in SIDES
             for rank_type, mrr in all_type_mrr.items():
                 assert rank_type in RANK_TYPES
                 assert isinstance(mrr, float)
                 assert 0 < mrr <= 1
+
+        # check hits at k (H@k)
         for side, all_type_hits_at_k in result.hits_at_k.items():
             assert side in SIDES
             for rank_type, hits_at_k in all_type_hits_at_k.items():
@@ -176,13 +75,33 @@ class RankBasedEvaluatorTests(_AbstractEvaluatorTests, unittest.TestCase):
                     assert isinstance(h, float)
                     assert 0 <= h <= 1
 
+        # check adjusted mean rank (AMR)
+        for side, adjusted_mean_rank in result.adjusted_arithmetic_mean_rank.items():
+            assert side in SIDES
+            assert RANK_REALISTIC in adjusted_mean_rank
+            assert isinstance(adjusted_mean_rank[RANK_REALISTIC], float)
+            assert 0 < adjusted_mean_rank[RANK_REALISTIC] < 2
+
+        # check adjusted mean rank index (AMRI)
+        for side, adjusted_mean_rank_index in result.adjusted_arithmetic_mean_rank_index.items():
+            assert side in SIDES
+            assert RANK_REALISTIC in adjusted_mean_rank_index
+            assert isinstance(adjusted_mean_rank_index[RANK_REALISTIC], float)
+            assert -1 <= adjusted_mean_rank_index[RANK_REALISTIC] <= 1
+
+        # the test only considered a single batch
+        for side, all_type_rank_counts in result.rank_count.items():
+            expected_size = 2 * self.batch_size if side == SIDE_BOTH else self.batch_size
+            # all rank types have the same count
+            assert set(all_type_rank_counts.values()) == {expected_size}
+
         # TODO: Validate with data?
 
 
-class SklearnEvaluatorTest(_AbstractEvaluatorTests, unittest.TestCase):
-    """Unittest for the SklearnEvaluator."""
+class ClassificationEvaluatorTest(cases.EvaluatorTestCase):
+    """Unittest for the ClassificationEvaluator."""
 
-    evaluator_cls = SklearnEvaluator
+    evaluator_cls = ClassificationEvaluator
 
     def _validate_result(
         self,
@@ -190,25 +109,34 @@ class SklearnEvaluatorTest(_AbstractEvaluatorTests, unittest.TestCase):
         data: Dict[str, torch.Tensor],
     ):
         # Check for correct class
-        assert isinstance(result, SklearnMetricResults)
+        assert isinstance(result, ClassificationMetricResults)
 
         # check value
-        scores = data['scores'].detach().cpu().numpy()
-        mask = data['mask'].detach().cpu().float().numpy()
+        scores = data["scores"].detach().cpu().numpy()
+        mask = data["mask"].detach().cpu().float().numpy()
+        batch = data["batch"].detach().cpu().numpy()
 
         # filtering
-        uniq = dict()
-        batch = data['batch'].detach().cpu().numpy()
-        for i, (h, r) in enumerate(batch[:, :2]):
-            uniq[int(h), int(r)] = i
-        indices = sorted(uniq.values())
-        mask = mask[indices]
-        scores = scores[indices]
+        mask_filtered, scores_filtered = [], []
+        for group_indices in [(0, 1), (1, 2)]:
+            uniq = dict()
+            for i, key in enumerate(batch[:, group_indices].tolist()):
+                uniq[tuple(key)] = i
+            indices = sorted(uniq.values())
+            mask_filtered.append(mask[indices])
+            scores_filtered.append(scores[indices])
+        mask = numpy.concatenate(mask_filtered, axis=0)
+        scores = numpy.concatenate(scores_filtered, axis=0)
 
-        for field in dataclasses.fields(SklearnMetricResults):
-            f = field.metadata['f']
-            exp_score = f(mask.flat, scores.flat)
-            self.assertAlmostEqual(result.get_metric(field.name), exp_score)
+        for field in sorted(dataclasses.fields(ClassificationMetricResults), key=attrgetter("name")):
+            with self.subTest(metric=field.name):
+                f = field.metadata["f"]
+                exp_score = f(numpy.array(mask.flat), numpy.array(scores.flat))
+                act_score = result.get_metric(field.name)
+                if numpy.isnan(exp_score):
+                    self.assertTrue(numpy.isnan(act_score))
+                else:
+                    self.assertAlmostEqual(act_score, exp_score, msg=f"failed for {field.name}", delta=7)
 
 
 class EvaluatorUtilsTests(unittest.TestCase):
@@ -221,34 +149,37 @@ class EvaluatorUtilsTests(unittest.TestCase):
     def test_compute_rank_from_scores(self):
         """Test the _compute_rank_from_scores() function."""
         batch_size = 3
-        all_scores = torch.tensor([
-            [2., 2., 1., 3., 5.],
-            [1., 1., 3., 4., 0.],
-            [1., 1., 3., float('nan'), 0],
-        ])
+        all_scores = torch.tensor(
+            [
+                [2.0, 2.0, 1.0, 3.0, 5.0],
+                [1.0, 1.0, 3.0, 4.0, 0.0],
+                [1.0, 1.0, 3.0, float("nan"), 0],
+            ]
+        )
         # true_score: (2, 3, 3)
-        true_score = torch.tensor([2., 3., 3.]).view(batch_size, 1)
-        exp_best_rank = torch.tensor([3., 2., 1.])
-        exp_worst_rank = torch.tensor([4., 2., 1.])
+        true_score = torch.as_tensor([2.0, 3.0, 3.0]).view(batch_size, 1)
+        exp_best_rank = torch.as_tensor([3.0, 2.0, 1.0])
+        exp_worst_rank = torch.as_tensor([4.0, 2.0, 1.0])
         exp_avg_rank = 0.5 * (exp_best_rank + exp_worst_rank)
-        exp_adj_rank = exp_avg_rank / torch.tensor([(5 + 1) / 2, (5 + 1) / 2, (4 + 1) / 2])
+        exp_exp_rank = torch.as_tensor([(5 + 1) / 2, (5 + 1) / 2, (4 + 1) / 2])
         ranks = compute_rank_from_scores(true_score=true_score, all_scores=all_scores)
 
-        best_rank = ranks.get('best')
-        assert best_rank.shape == (batch_size,)
-        assert (best_rank == exp_best_rank).all()
+        optimistic_rank = ranks.get(RANK_OPTIMISTIC)
+        assert optimistic_rank.shape == (batch_size,)
+        assert (optimistic_rank == exp_best_rank).all()
 
-        worst_rank = ranks.get('worst')
-        assert worst_rank.shape == (batch_size,)
-        assert (worst_rank == exp_worst_rank).all()
+        pessimistic_rank = ranks.get(RANK_PESSIMISTIC)
+        assert pessimistic_rank.shape == (batch_size,)
+        assert (pessimistic_rank == exp_worst_rank).all()
 
-        avg_rank = ranks.get('avg')
-        assert avg_rank.shape == (batch_size,)
-        assert (avg_rank == exp_avg_rank).all(), (avg_rank, exp_avg_rank)
+        realistic_rank = ranks.get(RANK_REALISTIC)
+        assert realistic_rank.shape == (batch_size,)
+        assert (realistic_rank == exp_avg_rank).all(), (realistic_rank, exp_avg_rank)
 
-        adj_rank = ranks.get('adj')
-        assert adj_rank.shape == (batch_size,)
-        assert (adj_rank == exp_adj_rank).all(), (adj_rank, exp_adj_rank)
+        expected_realistic_rank = ranks.get(RANK_EXPECTED_REALISTIC)
+        assert expected_realistic_rank is not None
+        assert expected_realistic_rank.shape == (batch_size,)
+        assert (expected_realistic_rank == exp_exp_rank).all(), (expected_realistic_rank, exp_exp_rank)
 
     def test_create_sparse_positive_filter_(self):
         """Test method create_sparse_positive_filter_."""
@@ -310,37 +241,43 @@ class EvaluatorUtilsTests(unittest.TestCase):
                 [0, 2, 2],
                 [3, 1, 2],
                 [1, 2, 0],
-            ], dtype=torch.long,
+            ],
+            dtype=torch.long,
         )
         batch = torch.tensor(
             [
                 [0, 1, 2],
                 [1, 2, 3],
-            ], dtype=torch.long,
+            ],
+            dtype=torch.long,
         )
         head_filter_mask = torch.tensor(
             [
                 [True, False, False, False],
                 [False, True, False, False],
-            ], dtype=torch.bool,
+            ],
+            dtype=torch.bool,
         )
         tail_filter_mask = torch.tensor(
             [
                 [False, False, True, False],
                 [False, False, False, True],
-            ], dtype=torch.bool,
+            ],
+            dtype=torch.bool,
         )
         exp_head_filter_mask = torch.tensor(
             [
                 [True, False, False, True],
                 [False, True, False, False],
-            ], dtype=torch.bool,
+            ],
+            dtype=torch.bool,
         )
         exp_tail_filter_mask = torch.tensor(
             [
                 [False, False, True, False],
                 [True, False, False, True],
-            ], dtype=torch.bool,
+            ],
+            dtype=torch.bool,
         )
         assert batch.shape == (batch_size, 3)
         assert head_filter_mask.shape == (batch_size, num_entities)
@@ -418,14 +355,25 @@ class DummyEvaluator(Evaluator):
 
     def finalize(self) -> MetricResults:  # noqa: D102
         return RankBasedMetricResults(
-            mean_rank=self.counter,
-            mean_reciprocal_rank=None,
-            adjusted_mean_rank=None,
+            arithmetic_mean_rank=self.counter,
+            geometric_mean_rank=None,
+            harmonic_mean_rank=None,
+            median_rank=None,
+            inverse_arithmetic_mean_rank=None,
+            inverse_geometric_mean_rank=None,
+            inverse_harmonic_mean_rank=None,
+            inverse_median_rank=None,
+            rank_std=None,
+            rank_var=None,
+            rank_mad=None,
+            rank_count=None,
+            adjusted_arithmetic_mean_rank=None,
+            adjusted_arithmetic_mean_rank_index=None,
             hits_at_k=dict(),
         )
 
     def __repr__(self):  # noqa: D105
-        return f'{self.__class__.__name__}(losses={self.losses})'
+        return f"{self.__class__.__name__}(losses={self.losses})"
 
 
 class TestEvaluationStructure(unittest.TestCase):
@@ -435,15 +383,94 @@ class TestEvaluationStructure(unittest.TestCase):
         """Prepare for testing the evaluation structure."""
         self.counter = 1337
         self.evaluator = DummyEvaluator(counter=self.counter, filtered=True, automatic_memory_optimization=False)
-        self.triples_factory = Nations().training
-        self.model = MockModel(triples_factory=self.triples_factory)
+        self.dataset = Nations()
+        self.model = FixedModel(triples_factory=self.dataset.training)
 
     def test_evaluation_structure(self):
         """Test if the evaluator has a balanced call of head and tail processors."""
         eval_results = self.evaluator.evaluate(
             model=self.model,
-            mapped_triples=self.triples_factory.mapped_triples,
+            additional_filter_triples=self.dataset.training.mapped_triples,
+            mapped_triples=self.dataset.testing.mapped_triples,
             batch_size=1,
             use_tqdm=False,
         )
-        assert eval_results.mean_rank == self.counter, 'Should end at the same value as it started'
+        self.assertIsInstance(eval_results, RankBasedMetricResults)
+        assert eval_results.arithmetic_mean_rank == self.counter, "Should end at the same value as it started"
+
+
+class TestEvaluationFiltering(unittest.TestCase):
+    """Tests for testing the correct filtering of positive triples of the evaluation procedure."""
+
+    def setUp(self):
+        """Prepare for testing the evaluation filtering."""
+        self.evaluator = RankBasedEvaluator(filtered=True, automatic_memory_optimization=False)
+        self.triples_factory = Nations().training
+        self.model = FixedModel(triples_factory=self.triples_factory)
+
+        # The MockModel gives the highest score to the highest entity id
+        max_score = self.triples_factory.num_entities - 1
+
+        # The test triples are created to yield the third highest score on both head and tail prediction
+        self.test_triples = torch.tensor([[max_score - 2, 0, max_score - 2]])
+
+        # Write new mapped triples to the model, since the model's triples will be used to filter
+        # These triples are created to yield the highest score on both head and tail prediction for the
+        # test triple at hand
+        self.training_triples = torch.tensor(
+            [
+                [max_score - 2, 0, max_score],
+                [max_score, 0, max_score - 2],
+            ],
+        )
+
+        # The validation triples are created to yield the second highest score on both head and tail prediction for the
+        # test triple at hand
+        self.validation_triples = torch.tensor(
+            [
+                [max_score - 2, 0, max_score - 1],
+                [max_score - 1, 0, max_score - 2],
+            ],
+        )
+
+    def test_evaluation_filtering_without_validation_triples(self):
+        """Test if the evaluator's triple filtering works as expected."""
+        eval_results = self.evaluator.evaluate(
+            model=self.model,
+            mapped_triples=self.test_triples,
+            additional_filter_triples=self.training_triples,
+            batch_size=1,
+            use_tqdm=False,
+        )
+        assert eval_results.arithmetic_mean_rank["both"]["realistic"] == 2, "The rank should equal 2"
+
+    def test_evaluation_filtering_with_validation_triples(self):
+        """Test if the evaluator's triple filtering works as expected when including additional filter triples."""
+        eval_results = self.evaluator.evaluate(
+            model=self.model,
+            mapped_triples=self.test_triples,
+            additional_filter_triples=[
+                self.training_triples,
+                self.validation_triples,
+            ],
+            batch_size=1,
+            use_tqdm=False,
+        )
+        assert eval_results.arithmetic_mean_rank["both"]["realistic"] == 1, "The rank should equal 1"
+
+
+def test_resolve_metric_name():
+    """Test metric name resolution."""
+    for name, expected in (
+        ("mrr", ("inverse_harmonic_mean_rank", "both", "realistic", None)),
+        ("mean_rank.both", ("arithmetic_mean_rank", "both", "realistic", None)),
+        ("mean_rank.avg", ("arithmetic_mean_rank", "both", "realistic", None)),
+        ("mean_rank.tail.worst", ("arithmetic_mean_rank", "tail", "pessimistic", None)),
+        ("amri.avg", ("adjusted_arithmetic_mean_rank_index", "both", "realistic", None)),
+        ("hits_at_k", ("hits_at_k", "both", "realistic", 10)),
+        ("hits_at_k.head.best.3", ("hits_at_k", "head", "optimistic", 3)),
+        ("hits_at_1", ("hits_at_k", "both", "realistic", 1)),
+        ("H@10", ("hits_at_k", "both", "realistic", 10)),
+    ):
+        result = resolve_metric_name(name=name)
+        assert result == expected, name
