@@ -6,11 +6,17 @@ import unittest
 
 import pytest
 import torch
+import unittest_templates
 from torch.nn import functional
 
 from pykeen.models import ConvKB, TransH
 from pykeen.regularizers import (
-    CombinedRegularizer, LpRegularizer, NoRegularizer, PowerSumRegularizer, Regularizer, TransHRegularizer,
+    CombinedRegularizer,
+    LpRegularizer,
+    NoRegularizer,
+    PowerSumRegularizer,
+    Regularizer,
+    TransHRegularizer,
 )
 from pykeen.utils import get_expected_norm, resolve_device
 from tests import cases
@@ -29,13 +35,13 @@ class NoRegularizerTest(cases.RegularizerTestCase):
 class L1RegularizerTest(cases.LpRegularizerTest):
     """Test an L_1 normed regularizer."""
 
-    kwargs = {'p': 1}
+    kwargs = {"p": 1}
 
 
 class NormedL2RegularizerTest(cases.LpRegularizerTest):
     """Test an L_2 normed regularizer."""
 
-    kwargs = {'normalize': True, 'p': 2}
+    kwargs = {"normalize": True, "p": 2}
 
     @pytest.mark.slow
     def test_expected_norm(self):
@@ -52,7 +58,7 @@ class NormedL2RegularizerTest(cases.LpRegularizerTest):
 
         # test error is raised
         with pytest.raises(NotImplementedError):
-            get_expected_norm(p=float('inf'), d=d)
+            get_expected_norm(p=float("inf"), d=d)
 
 
 class CombinedRegularizerTest(cases.RegularizerTestCase):
@@ -60,14 +66,14 @@ class CombinedRegularizerTest(cases.RegularizerTestCase):
 
     cls = CombinedRegularizer
     kwargs = {
-        'regularizers': [
+        "regularizers": [
             LpRegularizer(weight=0.1, p=1),
             LpRegularizer(weight=0.7, p=2),
         ],
     }
 
     def _expected_penalty(self, x: torch.FloatTensor) -> torch.FloatTensor:  # noqa: D102
-        regularizers = self.kwargs['regularizers']
+        regularizers = self.kwargs["regularizers"]
         return sum(r.weight * r.forward(x) for r in regularizers) / sum(r.weight for r in regularizers)
 
 
@@ -80,9 +86,9 @@ class PowerSumRegularizerTest(cases.RegularizerTestCase):
         kwargs = self.kwargs
         if kwargs is None:
             kwargs = {}
-        p = kwargs.get('p', self.instance.p)
+        p = kwargs.get("p", self.instance.p)
         value = x.pow(p).sum(dim=-1).mean()
-        if kwargs.get('normalize', False):
+        if kwargs.get("normalize", False):
             value = value / x.shape[-1]
         return value
 
@@ -105,7 +111,7 @@ class TransHRegularizerTest(unittest.TestCase):
         """Set up the test case."""
         self.generator = torch.random.manual_seed(seed=42)
         self.device = resolve_device()
-        self.kwargs = {'weight': .5, 'epsilon': 1e-5}
+        self.kwargs = {"weight": 0.5, "epsilon": 1e-5}
         self.instance = self.cls(
             **(self.kwargs or {}),
         ).to(self.device)
@@ -125,24 +131,24 @@ class TransHRegularizerTest(unittest.TestCase):
                 self.relations_weight,
                 rand(self.num_entities, 10, generator=self.generator, device=self.device),
             )
-            self.assertTrue('Expects exactly three tensors' in context.exception)
+            self.assertTrue("Expects exactly three tensors" in context.exception)
 
             self.instance.update(
                 self.entities_weight,
                 self.normal_vector_weight,
             )
-            self.assertTrue('Expects exactly three tensors' in context.exception)
+            self.assertTrue("Expects exactly three tensors" in context.exception)
 
         # Test that regularization term is computed correctly
         self.instance.update(self.entities_weight, self.normal_vector_weight, self.relations_weight)
         expected_term = self._expected_penalty()
-        weight = self.kwargs.get('weight')
+        weight = self.kwargs.get("weight")
         self.assertAlmostEqual(self.instance.term.item(), weight * expected_term.item())
 
     def _expected_penalty(self) -> torch.FloatTensor:  # noqa: D102
         # Entity soft constraint
         regularization_term = torch.sum(functional.relu(torch.norm(self.entities_weight, dim=-1) ** 2 - 1.0))
-        epsilon = self.kwargs.get('epsilon')  #
+        epsilon = self.kwargs.get("epsilon")  #
 
         # Orthogonality soft constraint
         d_r_n = functional.normalize(self.relations_weight, dim=-1)
@@ -166,8 +172,8 @@ class TestOnlyUpdateOnce(unittest.TestCase):
 
     def test_lp(self):
         """Test when the Lp regularizer only updates once, like for ConvKB."""
-        self.assertIn('apply_only_once', ConvKB.regularizer_default_kwargs)
-        self.assertTrue(ConvKB.regularizer_default_kwargs['apply_only_once'])
+        self.assertIn("apply_only_once", ConvKB.regularizer_default_kwargs)
+        self.assertTrue(ConvKB.regularizer_default_kwargs["apply_only_once"])
         regularizer = LpRegularizer(
             **ConvKB.regularizer_default_kwargs,
         )
@@ -175,7 +181,7 @@ class TestOnlyUpdateOnce(unittest.TestCase):
 
     def test_transh_regularizer(self):
         """Test the TransH regularizer only updates once."""
-        self.assertNotIn('apply_only_once', TransH.regularizer_default_kwargs)
+        self.assertNotIn("apply_only_once", TransH.regularizer_default_kwargs)
         regularizer = TransHRegularizer(
             **TransH.regularizer_default_kwargs,
         )
@@ -189,20 +195,14 @@ class TestOnlyUpdateOnce(unittest.TestCase):
         self.assertEqual(0.0, regularizer.regularization_term.item())
 
         # After first update, should change the term
-        first_tensors = [
-            rand(10, 10, generator=self.generator, device=self.device)
-            for _ in range(n_tensors)
-        ]
+        first_tensors = [rand(10, 10, generator=self.generator, device=self.device) for _ in range(n_tensors)]
         regularizer.update(*first_tensors)
         self.assertTrue(regularizer.updated)
         self.assertNotEqual(0.0, regularizer.regularization_term.item())
         term = regularizer.regularization_term.clone()
 
         # After second update, no change should happen
-        second_tensors = [
-            rand(10, 10, generator=self.generator, device=self.device)
-            for _ in range(n_tensors)
-        ]
+        second_tensors = [rand(10, 10, generator=self.generator, device=self.device) for _ in range(n_tensors)]
         regularizer.update(*second_tensors)
         self.assertTrue(regularizer.updated)
         self.assertEqual(term, regularizer.regularization_term)
@@ -212,7 +212,7 @@ class TestOnlyUpdateOnce(unittest.TestCase):
         self.assertEqual(0.0, regularizer.regularization_term.item())
 
 
-class TestRegularizerTests(cases.TestsTestCase):
+class TestRegularizerTests(unittest_templates.MetaTestCase[Regularizer]):
     """Test all regularizers are tested."""
 
     base_cls = Regularizer
